@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,6 +33,7 @@
 #include "grts/structs.app.h"
 
 #include "grt/editor_base.h"
+#include "grtui/grtdb_connect_dialog.h"
 
 #include "workbench/wb_context.h"
 #include "workbench/wb_context_ui.h"
@@ -59,6 +60,7 @@
 #include "mforms/code_editor.h"
 #include "mforms/menubar.h"
 #include "mforms/toolbar.h"
+#include "mforms/filechooser.h"
 
 using namespace wb;
 using namespace bec;
@@ -318,34 +320,6 @@ public:
   {
     _editor.reset();
   }
-
-
-  grt_PyObjectRef createCPyConnection()
-  {
-    boost::shared_ptr<SqlEditorForm> ref(_editor);
-
-    WillEnterPython lock;
-    grt::PythonContext *py = grt::PythonContext::get();
-    py->run_buffer("import mysql.connector");
-    PyObject *ctor = py->get_global("mysql.connector.Connect");
-    if (!ctor)
-      throw std::logic_error("Could not get handle to Connector method");
-
-    grt::AutoPyObject kwarg(PyDict_New());
-
-    grt::DictRef params(ref->connection_descriptor()->parameterValues());
-
-    PyDict_SetItemString(kwarg, "host", grt::AutoPyObject(PyString_FromString(params.get_string("hostName").c_str())));
-    PyDict_SetItemString(kwarg, "port", grt::AutoPyObject(PyLong_FromSize_t(params.get_int("port"))));
-    PyDict_SetItemString(kwarg, "user", grt::AutoPyObject(PyString_FromString(params.get_string("userName").c_str())));
-    PyDict_SetItemString(kwarg, "password", grt::AutoPyObject(PyString_FromString(ref->dbc_auth_data()->password())));
-
-    grt::AutoPyObject connection(PyObject_Call(ctor, grt::AutoPyObject(PyTuple_New(0)), kwarg));
-    if (!connection)
-      throw grt::python_error("error opening connection");
-
-    return pyobject_to_grt(_self->get_grt(), connection);
-  }
     
 protected:  
   db_query_Editor *_self;
@@ -418,13 +392,6 @@ static bool validate_export(wb::WBContextSQLIDE *sqlide)
   return has_active_resultset(sqlide);
 }
 
-static void call_run_file(wb::WBContextSQLIDE *sqlide)
-{
-  std::string path= sqlide->get_wbui()->get_wb()->show_file_dialog("open", _("Execute SQL Script"), "SQL Files (*.sql)|*.sql");
-  if (!path.empty())
-    sqlide->run_file(path);
-}
-
 static void call_save_file(wb::WBContextSQLIDE *sqlide)
 {
   SqlEditorForm *editor = sqlide->get_active_sql_editor();
@@ -515,13 +482,35 @@ static void call_new_connection(wb::WBContextSQLIDE *sqlide)
   }
 }
 
+static bool validate_has_connection(wb::WBContextSQLIDE *sqlide)
+{
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  return form && form->connection_descriptor().is_valid();
+}
+
+static void call_open_script(wb::WBContextSQLIDE *sqlide)
+{
+  mforms::FileChooser chooser(mforms::OpenFile);
+  chooser.set_title("Open SQL Script");
+  chooser.set_extensions("SQL Files (*.sql)|*.sql|Query Browser Files (*.qbquery)|*.qbquery", "sql");
+  if (chooser.run_modal())
+  {
+    boost::shared_ptr<SqlEditorForm> form = sqlide->get_wbui()->get_wb()->add_new_query_window();
+    if (form)
+    {
+      form->open_file(chooser.get_path());
+    }
+  }
+}
+
 
 static void call_exec_sql(wb::WBContextSQLIDE *sqlide, bool current_statement_only)
 {
   SqlEditorForm *form= sqlide->get_active_sql_editor();
   if (form)
     form->run_editor_contents(current_statement_only);
-}  
+}
+
 
 static bool validate_exec_sql(wb::WBContextSQLIDE *sqlide)
 {
@@ -649,7 +638,7 @@ static bool validate_toolbar_alias_toggle(wb::WBContextSQLIDE *sqlide, const std
 //--------------------------------------------------------------------------------------------------
 
 WBContextSQLIDE::WBContextSQLIDE(WBContextUI *wbui)
-: _wbui(wbui), _auto_save_active(false), _option_change_signal_connected(false)
+: _wbui(wbui), _auto_save_handle(0), _auto_save_active(false), _option_change_signal_connected(false)
 {
 }
 
@@ -657,6 +646,8 @@ WBContextSQLIDE::WBContextSQLIDE(WBContextUI *wbui)
 
 WBContextSQLIDE::~WBContextSQLIDE()
 {
+  if (_auto_save_handle)
+    mforms::Utilities::cancel_timeout(_auto_save_handle);
   base::NotificationCenter::get()->remove_observer(this);
 }
 
@@ -677,8 +668,11 @@ bool WBContextSQLIDE::auto_save_workspaces()
   WBContext *wb= _wbui->get_wb();
   ssize_t interval= wb->get_root()->options()->options().get_int("workbench:AutoSaveSQLEditorInterval", 60);
   if (interval <= 0 || !_auto_save_active)
+  {
+    _auto_save_handle = static_cast<mforms::TimeoutHandle>(NULL);
+    _auto_save_active = false;
     return false;
-
+  }
   for (std::list<boost::weak_ptr<SqlEditorForm> >::const_iterator iter = _open_editors.begin();
        iter != _open_editors.end(); ++iter)
   {
@@ -698,8 +692,11 @@ bool WBContextSQLIDE::auto_save_workspaces()
   
   if (interval != _auto_save_interval)
   {
+    _auto_save_interval = interval;
+    if (_auto_save_handle)
+      mforms::Utilities::cancel_timeout(_auto_save_handle);
     // schedule new interval
-    wb->get_grt_manager()->run_every(boost::bind(&WBContextSQLIDE::auto_save_workspaces, this), (double)interval);
+    _auto_save_handle = mforms::Utilities::add_timeout((float)interval, boost::bind(&WBContextSQLIDE::auto_save_workspaces, this));
     return false;
   }
   
@@ -780,6 +777,8 @@ void WBContextSQLIDE::init()
 
   cmdui->add_builtin_command("query.execute", boost::bind(call_exec_sql, this, false), boost::bind(validate_exec_sql, this));
   cmdui->add_builtin_command("query.execute_current_statement", boost::bind(call_exec_sql, this, true), boost::bind(validate_exec_sql, this));
+
+  cmdui->add_builtin_command("query.explain_current_statement", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::explain_current_statement), boost::bind(validate_exec_sql, this));
   
   cmdui->add_builtin_command("query.save_edits", boost::bind(call_save_edits, this), boost::bind(validate_save_edits, this));
   cmdui->add_builtin_command("query.discard_edits", boost::bind(call_discard_edits, this), boost::bind(validate_save_edits, this));
@@ -796,8 +795,11 @@ void WBContextSQLIDE::init()
   cmdui->add_builtin_command("query.new_routine", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_routine"));
   cmdui->add_builtin_command("query.new_function", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_function"));
 
-  cmdui->add_builtin_command("query.new_connection", boost::bind(call_new_connection, this));
-  
+  cmdui->add_builtin_command("query.new_connection", boost::bind(call_new_connection, this),
+                             boost::bind(validate_has_connection, this));
+
+  cmdui->add_builtin_command("query.openScriptNoConnection", boost::bind(call_open_script, this));
+
   cmdui->add_builtin_command("query.newQuery", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::new_scratch_area));
   //cmdui->add_builtin_command("query.newFile", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::new_sql_script_file));
   cmdui->add_builtin_command("query.newFile", boost::bind(new_script_tab, this));
@@ -806,8 +808,6 @@ void WBContextSQLIDE::init()
   cmdui->add_builtin_command("query.saveFileAs", boost::bind(call_save_file_as, this));
   cmdui->add_builtin_command("query.revert", boost::bind(call_revert, this), boost::bind(validate_revert, this));
 
-  cmdui->add_builtin_command("query.runFile", boost::bind(call_run_file, this));
-
   cmdui->add_builtin_command("query.export", boost::bind(call_export, this), boost::bind(validate_export, this));
   
   cmdui->add_builtin_command("query.cancel", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::cancel_query));
@@ -815,9 +815,6 @@ void WBContextSQLIDE::init()
   cmdui->add_builtin_command("query.reconnect", boost::bind(call_reconnect, this));
 
   cmdui->add_builtin_command("query.stopOnError", boost::bind(call_continue_on_error, this));
-  
-  cmdui->add_builtin_command("query.explain_current_statement",
-                                               boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::explain_current_statement));
 
   cmdui->add_builtin_command("query.jump_to_placeholder", boost::bind(&WBContextSQLIDE::call_in_editor_panel, this, &SqlEditorPanel::jump_to_placeholder));
   cmdui->add_builtin_command("list-members", boost::bind(&WBContextSQLIDE::call_in_editor_panel, this, &SqlEditorPanel::list_members),
@@ -827,7 +824,12 @@ void WBContextSQLIDE::init()
 
 void WBContextSQLIDE::finalize()
 {
-  std::list<SqlEditorForm::Ptr>::iterator next, ed = _open_editors.begin(); 
+  if (_auto_save_handle)
+  {
+    mforms::Utilities::cancel_timeout(_auto_save_handle);
+    _auto_save_handle = static_cast<mforms::TimeoutHandle>(NULL);
+  }
+  std::list<SqlEditorForm::Ptr>::iterator next, ed = _open_editors.begin();
   while (ed != _open_editors.end())
   {
     next = ed;
@@ -845,6 +847,17 @@ void WBContextSQLIDE::reconnect_editor(SqlEditorForm *editor)
 {
   boost::shared_ptr<sql::TunnelConnection> tunnel;
 
+  if (!editor->connection_descriptor().is_valid())
+  {
+    grtui::DbConnectionDialog dialog(get_wbui()->get_wb()->get_root()->rdbmsMgmt());
+    log_debug("No connection associated with editor on reconnect, showing connection selection dialog...\n");
+    db_mgmt_ConnectionRef target= dialog.run();
+    if (!target.is_valid())
+      return;
+
+    editor->set_connection(target);
+  }
+
   // open tunnel, if needed
   try
   {
@@ -933,54 +946,61 @@ static bool cancel_connect_editor(SqlEditorForm::Ref editor)
 SqlEditorForm::Ref WBContextSQLIDE::create_connected_editor(const db_mgmt_ConnectionRef &conn)
 {
   // start by opening the tunnel, if needed
-  boost::shared_ptr<sql::TunnelConnection> tunnel = sql::DriverManager::getDriverManager()->getTunnel(conn);
+  boost::shared_ptr<sql::TunnelConnection> tunnel;
+
+  if (conn.is_valid())
+    tunnel = sql::DriverManager::getDriverManager()->getTunnel(conn);
 
   SqlEditorForm::Ref editor(SqlEditorForm::create(this, conn));
 
-  void *result_ptr = 0;
-  if (!mforms::Utilities::run_cancelable_task(_("Opening SQL Editor"), 
-                                              strfmt(_("An SQL editor instance for '%s' is opening and should be available in a "
-                                                       "moment.\n\nPlease stand by..."), conn->name().c_str()),
-                                              boost::bind(connect_editor, editor, tunnel),
-                                              boost::bind(cancel_connect_editor, editor),
-                                              result_ptr))
-    throw grt::user_cancelled("canceled");
-  if (!result_ptr)
-    throw grt::user_cancelled("connection error");
-  
-  std::string *result = (std::string*)result_ptr;
-  if (result->empty())
-    delete result;
-  else
+  if (conn.is_valid())
   {
-    std::string tmp(*result);
-    delete result;
-    
-    if (tmp == ":PASSWORD_EXPIRED")
+    void *result_ptr = 0;
+    if (!mforms::Utilities::run_cancelable_task(_("Opening SQL Editor"),
+                                                strfmt(_("An SQL editor instance for '%s' is opening and should be available in a "
+                                                         "moment.\n\nPlease stand by..."), conn->name().c_str()),
+                                                boost::bind(connect_editor, editor, tunnel),
+                                                boost::bind(cancel_connect_editor, editor),
+                                                result_ptr))
+      throw grt::user_cancelled("canceled");
+    if (!result_ptr)
+      throw grt::user_cancelled("connection error");
+
+    std::string *result = (std::string*)result_ptr;
+    if (result->empty())
+      delete result;
+    else
     {
-      grt::BaseListRef args(conn->get_grt(), grt::AnyType);
-      args.ginsert(conn);
-      ssize_t result = *grt::IntegerRef::cast_from(conn->get_grt()->call_module_function("WbAdmin", "handleExpiredPassword", args));
-      if (result != 0)
-        return create_connected_editor(conn);
-      throw grt::user_cancelled("password reset cancelled by user");
+      std::string tmp(*result);
+      delete result;
+
+      if (tmp == ":PASSWORD_EXPIRED")
+      {
+        grt::BaseListRef args(conn->get_grt(), grt::AnyType);
+        args.ginsert(conn);
+        ssize_t result = *grt::IntegerRef::cast_from(conn->get_grt()->call_module_function("WbAdmin", "handleExpiredPassword", args));
+        if (result != 0)
+          return create_connected_editor(conn);
+        throw grt::user_cancelled("password reset cancelled by user");
+      }
+      else if (tmp == ":CANCELLED")
+      {
+        throw grt::user_cancelled("Cancelled");
+      }
+      
+      throw std::runtime_error(tmp);
     }
-    else if (tmp == ":CANCELLED")
-    {
-      throw grt::user_cancelled("Cancelled");
-    }
-    
-    throw std::runtime_error(tmp);
   }
 
   {
     // Create entry for grt tree and update volatile data in the connection.
     db_query_EditorRef object(_wbui->get_wb()->get_grt());
     object->owner(_wbui->get_wb()->get_root());
-    object->name(conn->name());
+    object->name(conn.is_valid() ? conn->name() : "unconnected");
     
     object->set_data(new db_query_EditorConcreteImplData(editor, object));
 
+    if (conn.is_valid())
     {
       std::map<std::string, std::string> details(editor->connection_details());
       grt::DictRef parameter_values = conn->parameterValues();
@@ -1004,7 +1024,7 @@ SqlEditorForm::Ref WBContextSQLIDE::create_connected_editor(const db_mgmt_Connec
     _auto_save_active= true;
     ssize_t interval = _wbui->get_wb()->get_root()->options()->options().get_int("workbench:AutoSaveSQLEditorInterval", 60);
     if (interval > 0)
-      _wbui->get_wb()->get_grt_manager()->run_every(boost::bind(&WBContextSQLIDE::auto_save_workspaces, this), (double)interval);
+        _auto_save_handle = mforms::Utilities::add_timeout((float)interval, boost::bind(&WBContextSQLIDE::auto_save_workspaces, this));
     _auto_save_interval = interval;
 
     if (!_option_change_signal_connected)
@@ -1013,13 +1033,15 @@ SqlEditorForm::Ref WBContextSQLIDE::create_connected_editor(const db_mgmt_Connec
       _option_change_signal_connected= true;
     }
   }
-  
-  if (::auto_save_sessions.find(conn.id()) != ::auto_save_sessions.end())
+
+  if (conn.is_valid())
   {
-    ::auto_save_sessions.erase(conn.id());
-    _wbui->refresh_home_connections();
+    if (::auto_save_sessions.find(conn.id()) != ::auto_save_sessions.end())
+    {
+      ::auto_save_sessions.erase(conn.id());
+      _wbui->refresh_home_connections();
+    }
   }
-  
   return editor;
 }
 
@@ -1050,26 +1072,12 @@ void WBContextSQLIDE::open_document(const std::string &path)
     editor->open_file(path);
   }
   else
-    mforms::Utilities::show_error(_("Open SQL Script"),
-                                  _("Please select a connected SQL Editor tab to open a script file."),
-                                  _("OK"));
-}
-
-
-void WBContextSQLIDE::run_file(const std::string &path)
-{
-  SqlEditorForm *editor= get_active_sql_editor();
-  if (editor)
   {
-    
+    boost::shared_ptr<SqlEditorForm> editor(get_wbui()->get_wb()->add_new_query_window());
+    editor->open_file(path);
   }
-  else
-    mforms::Utilities::show_error(_("Execute SQL Script"),
-                                  _("Please select a connected SQL Editor tab to run a script file."),
-                                  _("OK"));
 }
 
-
 static bool compare(SqlEditorForm::Ptr ptr, SqlEditorForm *editor)
 {
   return ptr.lock().get() == editor;
@@ -1165,9 +1173,9 @@ db_query_EditorRef WBContextSQLIDE::get_grt_editor_object(SqlEditorForm *editor)
 
 //--------------------------------------------------------------------------------------------------
 
-static struct RegisterNotifDocs
+static struct RegisterNotifDocs_wb_context_sqlide
 {
-  RegisterNotifDocs()
+  RegisterNotifDocs_wb_context_sqlide()
   {
     base::NotificationCenter::get()->register_notification("GRNSQLEditorOpened",
                                                            "sqlide",
@@ -1209,6 +1217,6 @@ static struct RegisterNotifDocs
                                                          "selected_rows - a list of int values of the selected rows\n"
                                                          "selected_column - if this value is set, the selection is a single cell at selected_rows[0], selected_column");
   }
-} initdocs;
+} initdocs_wb_context_sqlide;
 
 
