@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "wb_sql_editor_form.h"
 #include "wb_sql_editor_panel.h"
 #include "wb_sql_editor_result_panel.h"
@@ -44,6 +42,7 @@
 #include "objimpl/db.query/db_query_Resultset.h"
 
 #include "grtui/file_charset_dialog.h"
+#include "grtsqlparser/mysql_parser_services.h"
 
 #include <fstream>
 #include <sstream>
@@ -405,6 +404,10 @@ SqlEditorPanel::AutoSaveInfo::AutoSaveInfo(const std::string &info_file)
       word_wrap = value == "1";
     else if (key == "show_special")
       show_special = value == "1";
+    else if (key == "first_visible_line")
+      first_visible_line = base::atoi<int>(value, 0);
+    else if (key == "caret_pos")
+      caret_pos = base::atoi<int>(value, 0);
   }
 }
 
@@ -414,7 +417,7 @@ SqlEditorPanel::AutoSaveInfo SqlEditorPanel::AutoSaveInfo::old_scratch(const std
   AutoSaveInfo info;
   info.title = base::strip_extension(base::basename(scratch_file));
   if (base::is_number(info.title))
-    info.title = base::strfmt("Query %i", 1+atoi(info.title.c_str()));
+    info.title = base::strfmt("Query %i", 1 + base::atoi<int>(info.title, 0));
   info.type = "scratch";
   return info;
 }
@@ -453,7 +456,7 @@ bool SqlEditorPanel::load_autosave(const AutoSaveInfo &info,
     }
 
     // if this was a file, try to load it
-    if (!info.filename.empty() && !load_from(info.filename, info.orig_encoding, false))
+    if (!info.filename.empty() && load_from(info.filename, info.orig_encoding, false) != Loaded)
       return false;
   }
   else
@@ -466,7 +469,7 @@ bool SqlEditorPanel::load_autosave(const AutoSaveInfo &info,
     }
 
     // load the autosave
-    if (!load_from(text_file, info.orig_encoding, true))
+    if (load_from(text_file, info.orig_encoding, true) != Loaded)
       return false;
   }
   _filename = info.filename;
@@ -483,12 +486,15 @@ bool SqlEditorPanel::load_autosave(const AutoSaveInfo &info,
   item->set_checked(info.word_wrap);
   (*item->signal_activated())(item);
 
+  _editor->get_editor_control()->set_caret_pos(info.caret_pos);
+  _editor->get_editor_control()->send_editor(SCI_SETFIRSTVISIBLELINE, info.first_visible_line, 0);
+
   return true;
 }
 
 //--------------------------------------------------------------------------------------------------
 
-bool SqlEditorPanel::load_from(const std::string &file, const std::string &encoding, bool keep_dirty)
+SqlEditorPanel::LoadResult SqlEditorPanel::load_from(const std::string &file, const std::string &encoding, bool keep_dirty)
 {
   GError *error = NULL;
   gchar *data;
@@ -501,11 +507,14 @@ bool SqlEditorPanel::load_from(const std::string &file, const std::string &encod
     // auto completion.
     int result = mforms::Utilities::show_warning(_("Large File"),
                                                  strfmt(_("The file \"%s\" has a size "
-                                                                "of %.2f MB. Are you sure you want to open this large file?\n\nNote: code folding "
-                                                                "will be disabled for this file."), file.c_str(), file_size / 1024.0 / 1024.0),
-                                                 _("Open"), _("Cancel"));
-    if (result != mforms::ResultOk)
-      return false;
+                                                          "of %.2f MB. Are you sure you want to open this large file?\n\nNote: code folding "
+                                                          "will be disabled for this file.\n\nClick Run SQL Script... to just execute the file."),
+                                                        file.c_str(), file_size / 1024.0 / 1024.0),
+                                                 _("Open"), _("Cancel"), _("Run SQL Script..."));
+    if (result == mforms::ResultCancel)
+      return Cancelled;
+    else if (result == mforms::ResultOther)
+      return RunInstead;
   }
 
   _orig_encoding = encoding;
@@ -520,13 +529,20 @@ bool SqlEditorPanel::load_from(const std::string &file, const std::string &encod
 
   char *utf8_data;
   std::string original_encoding;
-  if (!FileCharsetDialog::ensure_filedata_utf8(_form->grt_manager()->get_grt(),
-                                               data, length, encoding, file,
-                                               utf8_data, &original_encoding))
+  FileCharsetDialog::Result result = FileCharsetDialog::ensure_filedata_utf8(_form->grt_manager()->get_grt(),
+                                                                             data, length, encoding, file,
+                                                                             utf8_data, &original_encoding);
+  if (result == FileCharsetDialog::Cancelled)
   {
     g_free(data);
-    return false;
+    return Cancelled;
   }
+  else if (result == FileCharsetDialog::RunInstead)
+  {
+    g_free(data);
+    return RunInstead;
+  }
+
   // if original data was in utf8, utf8_data comes back NULL
   if (!utf8_data)
     utf8_data = data;
@@ -553,8 +569,7 @@ bool SqlEditorPanel::load_from(const std::string &file, const std::string &encod
     log_warning("Can't get timestamp for %s\n", file.c_str());
     _file_timestamp = 0;
   }
-
-  return true;
+  return Loaded;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -640,7 +655,7 @@ bool SqlEditorPanel::save()
 void SqlEditorPanel::revert_to_saved()
 {
   _editor->sql("");
-  if (load_from(_filename, _orig_encoding))
+  if (load_from(_filename, _orig_encoding) == Loaded)
   {
     {
       NotificationInfo info;
@@ -689,6 +704,13 @@ void SqlEditorPanel::auto_save(const std::string &path)
       f << "word_wrap=1\n";
     else
       f << "word_wrap=0\n";
+
+    size_t caret_pos = _editor->get_editor_control()->get_caret_pos();
+    f << "caret_pos=" << caret_pos << "\n";
+
+    size_t first_line = _editor->get_editor_control()->send_editor(SCI_GETFIRSTVISIBLELINE, 0, 0);
+    f << "first_visible_line=" << first_line << "\n";
+
     f.close();
   }
 
@@ -802,9 +824,7 @@ mforms::ToolBar *SqlEditorPanel::setup_editor_toolbar()
   item->set_name("query.explain_current_statement");
   item->set_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_explain.png"));
   item->set_tooltip(_("Execute the EXPLAIN command on the statement under the cursor"));
-  _form->wbsql()->get_cmdui()->scoped_connect(item->signal_activated(),
-                                      boost::bind((void (wb::CommandUI::*)(const std::string&))&wb::CommandUI::activate_command,
-                                                  _form->wbsql()->get_cmdui(), "builtin:query.explain_current_statement"));
+  bec::UIForm::scoped_connect(item->signal_activated(),boost::bind(&SqlEditorForm::explain_current_statement, _form));
   tbar->add_item(item);
 
   item = mforms::manage(new mforms::ToolBarItem(mforms::ActionItem));
@@ -853,7 +873,7 @@ mforms::ToolBar *SqlEditorPanel::setup_editor_toolbar()
 
   item = mforms::manage(new mforms::ToolBarItem(mforms::SelectorItem));
   item->set_name("limit_rows");
-  item->set_tooltip(_("Set limit for number of rows returned by queries.\nWorkbech will automatically add the LIMIT clause with the configured number of rows to SELECT queries."));
+  item->set_tooltip(_("Set limit for number of rows returned by queries.\nWorkbench will automatically add the LIMIT clause with the configured number of rows to SELECT queries."));
   bec::UIForm::scoped_connect(item->signal_activated(), boost::bind(&SqlEditorPanel::limit_rows, this, item));
   tbar->add_item(item);
 
@@ -989,7 +1009,7 @@ void SqlEditorPanel::update_title()
 void SqlEditorPanel::list_members()
 {
   if (owner()->work_parser_context() != NULL)
-    editor_be()->show_auto_completion(true, owner()->work_parser_context()->recognizer());
+    editor_be()->show_auto_completion(false, owner()->work_parser_context());
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1031,7 +1051,7 @@ void SqlEditorPanel::query_started(bool retain_old_recordsets)
         int i = _lower_tabview.get_page_index(result);
         if (i >= 0)
         {
-          _lower_dock.close_view(result);
+          result->close();
           result_removed();
         }
       }
@@ -1126,6 +1146,10 @@ void SqlEditorPanel::lower_tab_switched()
     _tab_action_revert.set_enabled(false);
   }
 
+#ifdef _WIN32
+  _editor->focus();
+#endif
+
   mforms::MenuBar *menu;
   if ((menu = _form->get_menubar()))
   {
@@ -1185,13 +1209,27 @@ void SqlEditorPanel::on_recordset_context_menu_show(Recordset::Ptr rs_ptr)
 
 //--------------------------------------------------------------------------------------------------
 
-int SqlEditorPanel::result_count()
+/**
+ *	Returns the number of all docked result panels in the editor panel.
+ */
+size_t SqlEditorPanel::result_panel_count()
 {
   return _lower_tabview.page_count();
 }
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+*	Returns the number of all docked resultset panels in the editor panel.
+*	That excludes all the explain, spatial etc. panels.
+*/
+size_t SqlEditorPanel::resultset_count()
+{
+  return grtobj()->resultPanels().count();
+}
+
+//--------------------------------------------------------------------------------------------------
+
 SqlEditorResult *SqlEditorPanel::result_panel(int i)
 {
   if (i >= 0 && i < _lower_tabview.page_count())
@@ -1322,8 +1360,9 @@ bool SqlEditorPanel::lower_tab_closing(int tab)
   mforms::AppView *view = _lower_dock.view_at_index(tab);
   if (view)
   {
-    if (_lower_dock.close_view(view))
+    if (view->on_close())
     {
+      view->close();
       result_removed();
       return true;
     }
