@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -268,8 +268,8 @@ void WBContextUI::reset()
   _wb->request_refresh(RefreshSelection, "", 0);
 
   get_physical_overview()->send_refresh_children(bec::NodeId());
-  
-  _wb->get_component<WBComponentPhysical>()->update_catalog_tree_model();
+
+  _wb->get_model_context()->refill_catalog_tree();
 }
 
 
@@ -421,7 +421,7 @@ PhysicalOverviewBE *WBContextUI::get_physical_overview()
 void WBContextUI::show_output()
 {
   mforms::App::get()->dock_view(_output_view, "maintab");
-  mforms::App::get()->set_view_title(_output_view, "Output");
+  _output_view->set_title("Output");
   _output_view->setup_ui();
 }
 
@@ -437,7 +437,7 @@ void WBContextUI::show_web_page(const std::string& url, bool internal_browser)
   {
     WebBrowserView* browser= mforms::manage(new WebBrowserView(this));
     mforms::App::get()->dock_view(browser, "maintab");
-    mforms::App::get()->set_view_title(browser, _("Loading web page..."));
+    browser->set_title(_("Loading web page..."));
     browser->navigate(url);
   }
   else
@@ -952,14 +952,14 @@ void WBContextUI::set_wb_options_value(const std::string &model, const std::stri
   {
   case grt::DoubleType:
     {
-      grt::DoubleRef v(atof(value.c_str()));
+      grt::DoubleRef v(base::atof<double>(value, 0.0));
       if (!options.has_key(key) || options.get_double(key) != *v)
         options.set(key, v);
       break;
     }
   case grt::IntegerType:
     {
-      grt::IntegerRef v(atoi(value.c_str()));
+      grt::IntegerRef v(base::atoi<int>(value, 0));
       if (!options.has_key(key) || options.get_int(key) != *v)
         options.set(key, v);
       break;
@@ -1120,9 +1120,9 @@ bool WBContextUI::start_plugin_install(const std::string &path)
 
 //--------------------------------------------------------------------------------------------------
 
-static struct RegisterWBContextUINotifDocs
+static struct RegisterNotifDocs_wb_context_ui
 {
-  RegisterWBContextUINotifDocs()
+  RegisterNotifDocs_wb_context_ui()
   {
     base::NotificationCenter::get()->register_notification("GNAppStarted",
                                                            "application",
@@ -1149,6 +1149,6 @@ static struct RegisterWBContextUINotifDocs
                                                            "",
                                                            "");
   }
-} initdocs;
+} initdocs_wb_context_ui;
 
 
