@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,7 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-#include "stdafx.h"
 
 #include <glib/gstdio.h>
 
@@ -41,6 +40,7 @@
 
 #include "model/wb_context_model.h"
 #include "sqlide/wb_context_sqlide.h"
+#include "sqlide/wb_sql_editor_form.h"
 
 #include "new_server_instance_wizard.h"
 #include "server_instance_editor.h"
@@ -208,7 +208,7 @@ std::string get_server_info(db_mgmt_ServerInstanceRef instance)
   template <class T>
   void move_item_to_group(std::string group, grt::ListRef<T> items, const grt::ValueRef &object)
   {
-    size_t current_item_index, current_group_index, sibling_index, target_index;
+    size_t current_item_index = 0, current_group_index = 0, sibling_index = 0, target_index = 0;
     bool sibling_move = false;
     bool item_move = false;
     
@@ -290,6 +290,12 @@ void WBContextUI::show_about()
  */
 void WBContextUI::show_home_screen()
 {
+  if (_home_screen != NULL)
+  {
+    mforms::App::get()->select_view(_home_screen);
+    return;
+  }
+
   _initializing_home_screen = (_home_screen == NULL);
   if (_home_screen == NULL)
   {    
@@ -301,9 +307,27 @@ void WBContextUI::show_home_screen()
     // Setup context menus.
     mforms::Menu *menu;
 
+    {
+      std::list<std::string> groups;
+      bec::ArgumentPool argument_pool;
+      _wb->update_plugin_arguments_pool(argument_pool);
+      groups.push_back("Menu/Home/Connections");
+      bec::MenuItemList pitems = _wb->get_grt_manager()->get_plugin_context_menu_items(groups, argument_pool);
+      if (!pitems.empty())
+      {
+        menu = mforms::manage(new mforms::Menu());
+        menu->add_items_from_list(pitems);
+        _home_screen->set_menu(menu, HomeMenuConnectionGeneric);
+      }
+    }
+
     menu = mforms::manage(new mforms::Menu());
     menu->add_item(_("Open Connection"), "open_connection");
     menu->add_item(_("Edit Connection..."), "edit_connection");
+
+    menu->add_separator();
+    menu->add_item(_("Move to Group..."), "move_connection_to_group");
+
     {
       std::list<std::string> groups;
       bec::ArgumentPool argument_pool;
@@ -323,23 +347,52 @@ void WBContextUI::show_home_screen()
     menu->add_item(_("Move Up"), "move_connection_up");
     menu->add_item(_("Move Down"), "move_connection_down");
     menu->add_item(_("Move To End"), "move_connection_to_end");
-    menu->add_item(_("Move to Group..."), "move_connection_to_group");
 
     menu->add_separator();
     menu->add_item(_("Delete Connection..."), "delete_connection");
     menu->add_item(_("Delete All Connections..."), "delete_connection_all");
 
-    _home_screen->set_connection_menu(menu, false);
+    _home_screen->set_menu(menu, HomeMenuConnection);
 
     menu = mforms::manage(new mforms::Menu());
     menu->add_item(_("Move To Top"), "move_connection_to_top");
     menu->add_item(_("Move Up"), "move_connection_up");
     menu->add_item(_("Move Down"), "move_connection_down");
     menu->add_item(_("Move To End"), "move_connection_to_end");
+    menu->add_separator();
+    {
+      std::list<std::string> groups;
+      bec::ArgumentPool argument_pool;
+      _wb->update_plugin_arguments_pool(argument_pool);
+
+      argument_pool.add_simple_value("selectedGroupName", grt::StringRef(""));
+      groups.push_back("Menu/Home/ConnectionGroup");
+      bec::MenuItemList pitems = _wb->get_grt_manager()->get_plugin_context_menu_items(groups, argument_pool);
+      if (!pitems.empty())
+      {
+        menu->add_items_from_list(pitems);
+        menu->add_separator();
+      }
+    }
     menu->add_item(_("Delete Group..."), "delete_connection_group");
 
-    _home_screen->set_connection_menu(menu, true);
+    _home_screen->set_menu(menu, HomeMenuConnectionGroup);
 
+    menu = mforms::manage(new mforms::Menu());
+    menu->add_item(_("Move To Top"), "move_connection_to_top");
+    menu->add_item(_("Move Up"), "move_connection_up");
+    menu->add_item(_("Move Down"), "move_connection_down");
+    menu->add_item(_("Move To End"), "move_connection_to_end");
+
+    menu->add_separator();
+    menu->add_item(_("Edit Connection..."), "edit_connection");
+    menu->add_item(_("Delete Connection..."), "delete_fabric_connection");
+
+    menu->add_separator();
+    menu->add_item(_("Delete Connections to Managed Servers..."), "delete_fabric_connection_servers");
+
+    _home_screen->set_menu(menu, HomeMenuConnectionFabric);
+    
     menu = mforms::manage(new mforms::Menu());
     menu->add_item(_("Open Model"), "open_model_from_list");
     {
@@ -361,7 +414,7 @@ void WBContextUI::show_home_screen()
     menu->add_item(_("Show Model File"), "show_model");
     menu->add_item(_("Remove Model File from List"), "remove_model");
     menu->add_item(_("Clear List"), "remove_model_all");
-    _home_screen->set_document_menu(menu, true);
+    _home_screen->set_menu(menu, HomeMenuDocumentModel);
 
     menu = mforms::manage(new mforms::Menu());
     menu->add_item(_("Create EER Model from Database"), "model_from_schema");
@@ -381,7 +434,7 @@ void WBContextUI::show_home_screen()
       }      
     }
 
-    _home_screen->set_action_document_menu(menu, true);
+    _home_screen->set_menu(menu, HomeMenuDocumentModelAction);
   }
 
   mforms::App::get()->dock_view(_home_screen, "maintab");
@@ -413,13 +466,8 @@ void WBContextUI::show_home_screen()
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Called when the home screen is closed by the UI. We have to clear our reference then.
- */
 bool WBContextUI::home_screen_closing()
 {  
-  if (_home_screen)
-    _home_screen->release();
   _home_screen = NULL;
   return true;
 }
@@ -443,45 +491,16 @@ void WBContextUI::home_action_callback(HomeScreenAction action, const grt::Value
 //--------------------------------------------------------------------------------------------------
 
 /**
- * Removes a connection from the stored connections list along all associated data
+ * Removes a connection from the stored connections list along with all associated data
  * (including its server instance entry).
  */
 void WBContextUI::remove_connection(const db_mgmt_ConnectionRef &connection)
 {
-  grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
-  grt::ListRef<db_mgmt_ServerInstance> instances = _wb->get_root()->rdbmsMgmt()->storedInstances();
+  grt::GRT *grt = connection->get_grt();
+  grt::BaseListRef args(grt);
+  args->insert_unchecked(connection);
 
-  // Remove all associated server instances.
-  for (int i = instances.count()-1; i >= 0; --i)
-  {
-    db_mgmt_ServerInstanceRef instance(instances[i]);
-    if (instance->connection() == connection)
-      instances->remove(i);
-  }
-
-  // Remove password associated with this connection (if stored in keychain/vault). Check first
-  // this service/username combination isn't used anymore by other connections.
-  bool credentials_still_used = false;
-  grt::DictRef parameter_values = connection->parameterValues();
-  std::string host = connection->hostIdentifier();
-  std::string user = parameter_values.get_string("userName");
-  for (grt::ListRef<db_mgmt_Connection>::const_iterator i = connections.begin();
-    i != connections.end(); ++i)
-  {
-    if (*i != connection)
-    {
-      grt::DictRef current_parameters = (*i)->parameterValues();
-      if (host == *(*i)->hostIdentifier() && user == current_parameters.get_string("userName"))
-      {
-        credentials_still_used = true;
-        break;
-      }
-    }
-  }
-  if (!credentials_still_used)
-    mforms::Utilities::forget_password(host, user);
-
-  connections->remove(connection);
+  grt::ValueRef result = grt->call_module_function("Workbench", "deleteConnection", args);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -492,16 +511,37 @@ void WBContextUI::handle_home_context_menu(const grt::ValueRef &object, const st
   {
     handle_home_action(ActionOpenConnectionFromList, object);
   }
-  else if (action == "delete_connection")
+  else if (action == "delete_connection" || action == "delete_fabric_connection" || action == "delete_fabric_connection_servers")
   {
     db_mgmt_ConnectionRef connection(db_mgmt_ConnectionRef::cast_from(object));
     
     std::string name = connection->name();
-    std::string text= strfmt(_("Do you want to delete connection %s?"), name.c_str());
-    int answer = Utilities::show_warning(_("Delete Connection"), text,  _("Delete"), _("Cancel"));
+    std::string title;
+    std::string warning;
+
+    if (action == "delete_fabric_connection_servers")
+    {
+      title = _("Delete Managed Server Connections");
+      warning = strfmt(_("Do you really want to delete managed server connections on this fabric node: %s?"), name.c_str());
+    }
+    else
+    {
+      title = _("Delete Connection");
+      warning = strfmt(_("Do you want to delete connection %s?"), name.c_str());
+    }
+
+    int answer = Utilities::show_warning(title, warning,  _("Delete"), _("Cancel"));
     if (answer == mforms::ResultOk)
     {
-      remove_connection(connection);
+      // In case it is not a single connection, implies it is a fabric connection and we need to
+      // delete the connections to the managed servers
+      if (action != "delete_connection")
+        handle_home_context_menu(grt::StringRef(name), "internal_delete_connection_group");
+
+      // Connection is not deleted when we are just deleting the children of a fabric connection
+      if (action != "delete_fabric_connection_servers")
+        remove_connection(connection);
+
       refresh_home_connections();
     }
   }
@@ -513,31 +553,28 @@ void WBContextUI::handle_home_context_menu(const grt::ValueRef &object, const st
   {
     // We enter here for both: groups and connections. The object for groups must be a StringRef
     // with the name of the group. For connections it's the connection ref.
+    // Similar for the other move_* actions.
     grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
-    bec::move_list_ref_item<db_mgmt_Connection>(MoveTop, connections, object);
-    refresh_home_connections();
+    bec::move_list_ref_item<db_mgmt_Connection>(connections, object, MoveTop);
+    refresh_home_connections(false);
   }
   else if (action == "move_connection_up")
   {
-    // We enter here for both: groups and connections. The object for groups must be a StringRef
-    // with the name of the group. For connections it's the connection ref.
     grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
-    bec::move_list_ref_item<db_mgmt_Connection>(MoveUp, connections, object);
-    refresh_home_connections();
+    bec::move_list_ref_item<db_mgmt_Connection>(connections, object, MoveUp);
+    refresh_home_connections(false);
   }
   else if (action == "move_connection_down")
   {
     grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
-    bec::move_list_ref_item<db_mgmt_Connection>(MoveDown, connections, object);
-
-    refresh_home_connections();
+    bec::move_list_ref_item<db_mgmt_Connection>(connections, object, MoveDown);
+    refresh_home_connections(false);
   }
   else if (action == "move_connection_to_end")
   {
     grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
-    bec::move_list_ref_item<db_mgmt_Connection>(MoveBottom, connections, object);
-
-    refresh_home_connections();
+    bec::move_list_ref_item<db_mgmt_Connection>(connections, object, MoveBottom);
+    refresh_home_connections(false);
   }
   else if (action == "move_connection_to_group")
   {
@@ -559,34 +596,30 @@ void WBContextUI::handle_home_context_menu(const grt::ValueRef &object, const st
       refresh_home_connections();
     }
   }
-  else if (action == "delete_connection_group")
+  else if (action == "delete_connection_group" || action == "internal_delete_connection_group")
   {
     std::string group = object.repr();
-    std::string text= strfmt(_("Do you really want to delete all the connections in group: %s?"), group.c_str());
-    int answer = Utilities::show_warning(_("Delete Connection Group"), text,  _("Delete"), _("Cancel"));
+    int answer = mforms::ResultOk;
+    
+
+    // Internal deletion does not require the prompt
+    if (action == "delete_connection_group")
+    {
+      std::string text= strfmt(_("Do you really want to delete all the connections in group: %s?"), base::left(group, (unsigned int)group.length() -1).c_str());
+      answer = Utilities::show_warning(_("Delete Connection Group"), text,  _("Delete"), _("Cancel"));
+    }
+    
     if (answer == mforms::ResultOk)
     {
-      group += "/";
-      int group_length = group.length();
+      grt::GRT *grt = _wb->get_grt();
+      grt::BaseListRef args(grt);
+      args->insert_unchecked(object);
 
-      std::vector<db_mgmt_ConnectionRef> candidates;
-      grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
+      grt::ValueRef result = grt->call_module_function("Workbench", "deleteConnectionGroup", args);
 
-      int index = connections.count() - 1;
-      while (index >= 0)
-      {
-        std::string name = connections[index]->name();
-
-        if (name.compare(0, group_length, group) == 0)
-          candidates.push_back(connections[index]);
-
-        index--;
-      }
-
-      for (std::vector<db_mgmt_ConnectionRef>::const_iterator iterator = candidates.begin();
-        iterator != candidates.end(); ++iterator)
-        remove_connection(*iterator);
-      refresh_home_connections();
+      // Internal deletion does not require the UI update
+      if (action == "delete_connection_group")
+        refresh_home_connections();
     }
   }
   else if (action == "delete_connection_all")
@@ -665,7 +698,7 @@ void WBContextUI::handle_home_context_menu(const grt::ValueRef &object, const st
     if (answer == mforms::ResultOk)
     {
       grt::StringListRef file_names(_wb->get_root()->options()->recentFiles());
-      for (int index= file_names->count() - 1; index >= 0; index--)
+      for (ssize_t index = file_names->count() - 1; index >= 0; index--)
       {
         if (g_str_has_suffix(file_names[index].c_str(), ".mwb"))
           file_names->remove(index);
@@ -686,9 +719,15 @@ void WBContextUI::handle_home_context_menu(const grt::ValueRef &object, const st
     else
       if (grt::StringRef::can_wrap(object))
       {
-        argument_pool.add_simple_value("selectedModelFile", grt::StringRef::cast_from(object));
+        grt::StringRef arg(grt::StringRef::cast_from(object));
+        if (has_suffix(arg, ".mwb"))
+          argument_pool.add_simple_value("selectedModelFile", arg); // assume a model file
+        else
+          argument_pool.add_simple_value("selectedGroupName", arg); // assume a connection group name
         get_command_ui()->activate_command(action, argument_pool);
       }
+      else // Any other command.
+        get_command_ui()->activate_command(action, argument_pool);
   }
 }
 
@@ -752,15 +791,41 @@ void WBContextUI::handle_home_action(HomeScreenAction action, const grt::ValueRe
         break;
       _processing_action_open_connection = true;
       if (object.is_valid())
-      {        
+      {
         db_mgmt_ConnectionRef connection(db_mgmt_ConnectionRef::cast_from(object));
-        _wb->show_status_text("Opening SQL Editor...");          
+
+        _wb->show_status_text("Opening SQL Editor...");
         _wb->add_new_query_window(connection);
       }
       _processing_action_open_connection = false;
       break;
     }
       
+    case ActionFilesWithConnection:
+      {
+        if (_processing_action_open_connection)
+          break;
+
+        _processing_action_open_connection = true;
+        if (object.is_valid())
+        {
+          grt::DictRef dict = grt::DictRef::cast_from(object);
+          db_mgmt_ConnectionRef connection = db_mgmt_ConnectionRef::cast_from(dict["connection"]);
+          grt::StringListRef names = grt::StringListRef::cast_from(dict["files"]);
+          _wb->show_status_text("Opening files in new SQL Editor ...");          
+          boost::shared_ptr<SqlEditorForm> form = _wb->add_new_query_window(connection, false);
+          if (form)
+          {
+            for (size_t i = 0; i < names->count(); ++i)
+              form->open_file(names[i], true);
+            form->update_title();
+          }
+        }
+
+        _processing_action_open_connection = false;
+        break;
+      }
+
     case ActionNewConnection:
     {
       NewConnectionWizard wizard(_wb, _wb->get_root()->rdbmsMgmt());
@@ -780,6 +845,88 @@ void WBContextUI::handle_home_action(HomeScreenAction action, const grt::ValueRe
       break;
     }
 
+    case ActionMoveConnection:
+    {
+      grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
+
+      grt::DictRef dict = grt::DictRef::cast_from(object);
+      int to = (int)grt::IntegerRef::cast_from(dict["to"]);
+      if (db_mgmt_ConnectionRef::can_wrap(dict["object"]))
+      {
+        db_mgmt_ConnectionRef connection = db_mgmt_ConnectionRef::cast_from(dict["object"]);
+        move_list_ref_item<db_mgmt_Connection>(connections, connection, to);
+        refresh_home_connections(false);
+      }
+      else
+      {
+        if (grt::StringRef::can_wrap(dict["object"]))
+        {
+          grt::StringRef group = grt::StringRef::cast_from(dict["object"]);
+          move_list_ref_item<db_mgmt_Connection>(connections, group, to);
+          refresh_home_connections(false);
+        }
+
+      }
+      break;
+    }
+
+    case ActionMoveConnectionToGroup:
+    {
+      grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
+      db_mgmt_ConnectionRef connection;
+
+      grt::DictRef dict = grt::DictRef::cast_from(object);
+      std::string group = grt::StringRef::cast_from(dict["group"]);
+      connection = db_mgmt_ConnectionRef::cast_from(dict["object"]);
+      if (group != "" && connection.is_valid())
+      {
+        move_item_to_group<db_mgmt_Connection>(group, connections, connection);
+        refresh_home_connections(false);
+      }
+      break;
+    }
+    
+    case ActionUpdateFabricConnections:
+    {
+      // Creates the fabric connections only if they have not been already created
+      // since the last connection refresh
+      GUILock lock(_wb, _("Connecting to MySQL Fabric Management Node"), _("The connections to the managed MySQL Servers will be updated in a moment.\n"
+                                                                     "\nPlease stand by..."));
+      _wb->show_status_text(_("Connecting to MySQL Fabric Management Node..."));
+      db_mgmt_ConnectionRef connection(db_mgmt_ConnectionRef::cast_from(object));
+      grt::GRT *grt = connection->get_grt();
+      grt::BaseListRef args(grt);
+      args->insert_unchecked(connection);
+      
+      std::string error;
+
+      try
+      {
+        grt::ValueRef result = grt->call_module_function("WBFabric", "updateConnections", args);
+        error = grt::StringRef::extract_from(result);
+      }
+      catch (std::runtime_error &exc)
+      {
+        error = exc.what();
+      }
+      
+      if (error == "Operation Cancelled")
+        _wb->show_status_text(_("Creation of connections to Managed MySQL Servers was cancelled."));
+      else
+      {
+        if (error.length())
+        {
+          log_warning("Error updating managed connections for %s: %s\n", connection->name().c_str(), error.c_str());
+          mforms::Utilities::show_error(_("MySQL Fabric Connection Error"), error, "OK");
+          _wb->show_status_text(_("Failed updating connections to Managed MySQL Servers..."));
+        }
+        else
+          _wb->show_status_text(_("Updated connections to Managed MySQL Servers..."));
+      }
+
+      break;
+    }
+      
     case ActionSetupRemoteManagement:
     {
       NewServerInstanceWizard wizard(_wb, db_mgmt_ConnectionRef::cast_from(object));
@@ -878,7 +1025,7 @@ void WBContextUI::handle_home_action(HomeScreenAction action, const grt::ValueRe
 
 //--------------------------------------------------------------------------------------------------
 
-void WBContextUI::refresh_home_connections()
+void WBContextUI::refresh_home_connections(bool clear_state)
 {
   if (!_home_screen)
     return;
@@ -886,8 +1033,8 @@ void WBContextUI::refresh_home_connections()
   grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
 
   std::map<std::string, std::string> auto_save_files = WBContextSQLIDE::auto_save_sessions();
-  
-  _home_screen->clear_connections();
+
+  _home_screen->clear_connections(clear_state);
 
   // If there are no connections defined yet then create entries for all currently installed
   // local servers (only if this is the first run, after application start).
@@ -902,33 +1049,60 @@ void WBContextUI::refresh_home_connections()
     module->call_function("createInstancesFromLocalServers", arguments);
   }
 
-  for (grt::ListRef<db_mgmt_Connection>::const_iterator end = connections.end(),
-       inst = connections.begin(); inst != end; ++inst)
-  {
-    grt::DictRef dict((*inst)->parameterValues());
+  std::vector<db_mgmt_ConnectionRef> invalid_connections;
+  std::map<std::string, std::string> invalid_connection_ids;
 
-    std::string host_entry;
-    if ((*inst)->driver().is_valid() && (*inst)->driver()->name() == "MysqlNativeSSH")
-      host_entry = dict.get_string("sshUserName") + "@" + dict.get_string("sshHost");
+  for (grt::ListRef<db_mgmt_Connection>::const_iterator end = connections.end(),
+      inst = connections.begin(); inst != end; ++inst)
+  {
+    // Any connection with NULL driver will be considered invalid and so deleted
+    if (!(*inst)->driver().is_valid())
+    {
+      invalid_connections.push_back(*inst);
+      invalid_connection_ids.insert(std::pair<std::string, std::string>((*inst)->id(), ""));
+    }
     else
     {
-      host_entry = strfmt("%s:%i", dict.get_string("hostName").c_str(), (int) dict.get_int("port"));
-      if ((*inst)->driver().is_valid() && (*inst)->driver()->name() == "MysqlNativeSocket")
+      grt::DictRef dict((*inst)->parameterValues());
+
+      // Any fabric managed connection referencing a 
+      if (dict.has_key("fabric_managed") && invalid_connection_ids.count(dict.get_string("fabric_managed")))
+        invalid_connections.push_back(*inst);
+      else
       {
-        // TODO: what about the default for sockets (only have a default for the pipe name)?
-        std::string socket= dict.get_string("socket", "MySQL"); // socket or pipe
-        host_entry= "Localhost via pipe " + socket;
+        std::string host_entry;
+        if ((*inst)->driver().is_valid() && (*inst)->driver()->name() == "MysqlNativeSSH")
+          host_entry = dict.get_string("sshUserName") + "@" + dict.get_string("sshHost");
+        else
+        {
+          host_entry = strfmt("%s:%i", dict.get_string("hostName").c_str(), (int)dict.get_int("port", 3306));
+          if ((*inst)->driver().is_valid() && (*inst)->driver()->name() == "MysqlNativeSocket")
+          {
+            // TODO: what about the default for sockets (only have a default for the pipe name)?
+            std::string socket = dict.get_string("socket", "MySQL"); // socket or pipe
+            host_entry = "Localhost via pipe " + socket;
+          }
+        }
+
+        std::string title = *(*inst)->name();
+        if (auto_save_files.find((*inst)->id()) != auto_save_files.end())
+          title += " (auto saved)";
+
+        _home_screen->add_connection(*inst, title, host_entry,
+          dict.get_string("userName"), dict.get_string("schema"));
       }
     }
-
-    std::string title = *(*inst)->name();
-    if (auto_save_files.find((*inst)->id()) != auto_save_files.end())
-      title += " (auto saved)";
-
-    _home_screen->add_connection(*inst, title, host_entry,
-      dict.get_string("userName"), dict.get_string("schema"));
   }
-  
+
+
+  // Deletes the fabric managed connections
+  for (std::vector<db_mgmt_ConnectionRef>::const_iterator iterator = invalid_connections.begin();
+    iterator != invalid_connections.end(); ++iterator)
+  {
+    log_warning("Invalid connection detected %s, deleting it\n", (*iterator)->name().c_str());
+    remove_connection(*iterator);
+  }
+
   _wb->save_connections();
   _wb->save_instances();
 }
@@ -998,7 +1172,6 @@ void WBContextUI::refresh_home_documents()
       for (std::list<std::string>::const_iterator path_iterator = examples_paths.begin();
         path_iterator != examples_paths.end(); path_iterator++)
       {
-        g_message("%s", path_iterator->c_str());
         if (g_file_test(path_iterator->c_str(), G_FILE_TEST_IS_DIR))
         {
           std::string pattern = make_path(*path_iterator, "*.mwb");
@@ -1015,14 +1188,6 @@ void WBContextUI::refresh_home_documents()
   for (grt::StringListRef::const_iterator end = recentFiles.end(), f = recentFiles.begin();
     f != end; ++f)
   {
-    bool has_auto_save;
-    
-    // Check if there's an auto saved version of the file.
-    if (auto_save_models.find(*f) != auto_save_models.end()
-        || auto_save_models.find(base::basename((*f))) != auto_save_models.end())
-      has_auto_save = true;
-    else
-      has_auto_save = false;
     
 #ifdef _WIN32
     struct _stat stbuf;
