@@ -105,7 +105,7 @@ std::string WorkbenchImpl::getSystemInfo(bool indent)
   app_InfoRef info(app_InfoRef::cast_from(_wb->get_grt()->get("/wb/info")));
 
   const char* tab = indent ? "\t" : "";
-  std::string result = strfmt("%s%s %s (%s) for "PLATFORM_NAME" version %i.%i.%i %s revision %i build %i (%s)\n",
+  std::string result = strfmt("%s%s %s (%s) for " PLATFORM_NAME " version %i.%i.%i %s revision %i build %i (%s)\n",
     tab, info->name().c_str(), APP_EDITION_NAME, APP_LICENSE_TYPE, APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER,
                               APP_RELEASE_TYPE, APP_REVISION_NUMBER, APP_BUILD_NUMBER, ARCHITECTURE);
   result += strfmt("%sConfiguration Directory: %s\n", tab, _wb->get_grt_manager()->get_user_datadir().c_str());
@@ -153,6 +153,51 @@ std::string WorkbenchImpl::getSystemInfo(bool indent)
       g_free(stdo);
     }
   }
+
+  //try to find out if we're running in fips mode
+  {
+
+    bool fips_crypto = false;
+    {
+      std::ifstream fips_check;
+      fips_check.open("/proc/sys/crypto/fips_enabled");
+      if (fips_check.good())
+      {
+        char val;
+        fips_check >> val;
+        fips_crypto = val == '1' ? true : false;
+      }
+    }
+    bool fips_kernel = false;
+    {
+      std::ifstream fips_check;
+      fips_check.open("/proc/cmdline");
+      if (fips_check.good())
+      {
+        std::string line;
+        fips_check >> line;
+        std::size_t found = line.find("fips=");
+        if (found != std::string::npos)
+        {
+          if (found + 5 <= line.size() && line.substr(found + 5, 1) == "1")
+            fips_kernel = 1;
+        }
+      }
+    }
+    result += strfmt("%sFips mode enabled: %s\n", tab, (fips_kernel || fips_crypto) ? "yes" : "no");
+
+    // get env variables
+    {
+      int rc;
+      char *stdo;
+      if (g_spawn_command_line_sync("/usr/bin/env", &stdo, NULL, &rc, NULL) && stdo)
+      {
+        log_debug3("Environment variables:\n %s\n", stdo);
+        g_free(stdo);
+      }
+    }
+
+  }
 #endif
 
   return result;
@@ -200,7 +245,7 @@ std::map<std::string, std::string> WorkbenchImpl::getSystemInfoMap()
 #define def_plugin(group, aName, type, aCaption, descr)\
 {\
   app_PluginRef plugin(get_grt());\
-  plugin->name("wb."group"."aName);\
+  plugin->name("wb." group "." aName);\
   plugin->caption(aCaption);\
   plugin->description(descr);\
   plugin->moduleName("Workbench");\
@@ -213,7 +258,7 @@ std::map<std::string, std::string> WorkbenchImpl::getSystemInfoMap()
 #define def_object_plugin(group, klass, aName, aCaption, descr)\
 {\
   app_PluginRef plugin(get_grt());\
-  plugin->name("wb."group"."aName);\
+  plugin->name("wb." group "." aName);\
   plugin->caption(aCaption);\
   plugin->description(descr);\
   plugin->moduleName("Workbench");\
@@ -231,7 +276,7 @@ std::map<std::string, std::string> WorkbenchImpl::getSystemInfoMap()
 #define def_view_plugin(group, aName, aCaption, descr)\
 {\
   app_PluginRef plugin(get_grt());\
-  plugin->name("wb."group"."aName);\
+  plugin->name("wb." group "." aName);\
   plugin->caption(aCaption);\
   plugin->description(descr);\
   plugin->moduleName("Workbench");\
@@ -249,7 +294,7 @@ std::map<std::string, std::string> WorkbenchImpl::getSystemInfoMap()
 #define def_model_plugin(group, aName, aCaption, descr)\
 {\
   app_PluginRef plugin(get_grt());\
-  plugin->name("wb."group"."aName);\
+  plugin->name("wb." group "." aName);\
   plugin->caption(aCaption);\
   plugin->description(descr);\
   plugin->moduleName("Workbench");\
@@ -267,7 +312,7 @@ std::map<std::string, std::string> WorkbenchImpl::getSystemInfoMap()
 #define def_form_model_plugin(group, aName, aCaption, descr)\
 {\
   app_PluginRef plugin(get_grt());\
-  plugin->name("wb."group"."aName);\
+  plugin->name("wb." group "." aName);\
   plugin->caption(aCaption);\
   plugin->description(descr);\
   plugin->moduleName("Workbench");\
@@ -285,7 +330,7 @@ std::map<std::string, std::string> WorkbenchImpl::getSystemInfoMap()
 #define def_form_plugin(group, aName, aCaption, descr)\
 {\
   app_PluginRef plugin(get_grt());\
-  plugin->name("wb."group"."aName);\
+  plugin->name("wb." group "." aName);\
   plugin->caption(aCaption);\
   plugin->description(descr);\
   plugin->moduleName("Workbench");\
@@ -300,7 +345,7 @@ std::map<std::string, std::string> WorkbenchImpl::getSystemInfoMap()
 {\
   app_PluginRef plugin(get_grt());\
   app_PluginInputDefinitionRef pdef(get_grt());\
-  plugin->name("wb."group"."aName);\
+  plugin->name("wb." group "." aName);\
   plugin->caption(aCaption);\
   plugin->description(descr);\
   plugin->moduleName("Workbench");\
@@ -316,7 +361,7 @@ std::map<std::string, std::string> WorkbenchImpl::getSystemInfoMap()
 #define def_model_arg_plugin(group, aName, type, aCaption, descr)\
 {\
   app_PluginRef plugin(get_grt());\
-  plugin->name("wb."group"."aName);\
+  plugin->name("wb." group "." aName);\
   plugin->caption(aCaption);\
   plugin->description(descr);\
   plugin->moduleName("Workbench");\
@@ -339,7 +384,7 @@ std::map<std::string, std::string> WorkbenchImpl::getSystemInfoMap()
 {\
   app_PluginRef plugin(get_grt());\
   app_PluginFileInputRef pdef(get_grt());\
-  plugin->name("wb."group"."aName);\
+  plugin->name("wb." group "." aName);\
   plugin->caption(aCaption);\
   plugin->description(descr);\
   plugin->moduleName("Workbench");\
@@ -482,7 +527,7 @@ int WorkbenchImpl::openModel(const std::string &filename)
 
 int WorkbenchImpl::openRecentModel(const std::string &index)
 {
-  _wb->open_recent_document(atoi(index.c_str()));
+  _wb->open_recent_document(base::atoi<int>(index, 0));
 
   return 0;
 }
@@ -685,8 +730,6 @@ int WorkbenchImpl::editSelectedFigure(const model_DiagramRef &view)
 int WorkbenchImpl::editSelectedFigureInNewWindow(const model_DiagramRef &view)
 {
   ModelDiagramForm *form= dynamic_cast<ModelDiagramForm*>(_wb->get_active_form());
-  if (!form)
-    form = dynamic_cast<ModelDiagramForm*>(_wb->get_active_main_form());
   if (form)
   {
     ListRef<model_Object> list(form->get_selection());
@@ -1814,7 +1857,7 @@ grt::DictListRef WorkbenchImpl::getLocalServerList()
     char *ster = NULL;
     int rc=0;
     GError *err = NULL;
-    if (g_spawn_command_line_sync("/bin/bash -c \"ps -ec | grep \\\"mysqld\\b\\\" | awk '{ print $1 }' | xargs ps -fp | awk 'NR > 1 {for(i=1;i<=7;i++)$i=\\\"\\\"; print $0}'\"", &stdo, &ster, &rc, &err) && stdo)
+    if (g_spawn_command_line_sync("/bin/sh -c \"ps -ec | grep \\\"mysqld\\b\\\" | awk '{ print $1 }' | xargs -r ps -ww -o args= -p \"", &stdo, &ster, &rc, &err) && stdo)
     {
       std::string processes(stdo);
       
@@ -1838,15 +1881,14 @@ grt::DictListRef WorkbenchImpl::getLocalServerList()
   
     if (err)
     {
-      log_error("Error looking for installed servers, error %d : %s\n", err->code, err->message);
+      log_warning("Error looking for installed servers, error %d : %s\n", err->code, err->message);
       g_error_free(err);
     }
     
-    if (ster)
-    {
+    if (ster != NULL && strlen(ster) > 0)
       log_error("stderr from process list %s\n", ster);
-      g_free(ster);
-    }
+
+    g_free(ster);
     
 #endif
 
@@ -1918,7 +1960,7 @@ int WorkbenchImpl::createConnectionsFromLocalServers()
  */
 int WorkbenchImpl::createInstancesFromLocalServers()
 {
-
+  int found_instances = 0;
   try
   {
     grt::DictListRef servers = getLocalServerList();
@@ -1944,7 +1986,7 @@ int WorkbenchImpl::createInstancesFromLocalServers()
     
       ssize_t port = INT_MIN;
       if (str_port.length())
-        port = atoi(str_port.c_str());
+        port = base::atoi<int>(str_port, 0);
       
       bool can_use_networking = true;
       if (path.find("--skip-networking") != std::string::npos)
@@ -2072,10 +2114,11 @@ int WorkbenchImpl::createInstancesFromLocalServers()
       // If we did not find a connection for this instance then create a new one.
       if (!connection.is_valid())
         connection = create_connection("localhost", "root", socket_or_pipe_name, can_use_networking,
-        can_use_socket_or_pipe, (int)port, _("Local instance ") + display_name);
+                                       can_use_socket_or_pipe, (int)port, _("Local instance ") + display_name);
 
       instance->connection(connection);
       instances.insert(instance);
+      ++found_instances;
     }
   }
   catch (std::exception &exc)
@@ -2085,7 +2128,7 @@ int WorkbenchImpl::createInstancesFromLocalServers()
     log_warning("Error auto-detecting server instance: %s\n", exc.what());
   }
 
-  return 0;
+  return found_instances;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -2169,9 +2212,9 @@ std::string WorkbenchImpl::getFullVideoAdapterInfo(bool indent)
     {
       result << tab << "Active video adapter " << entry.get_string("Caption", "unknown") << '\n';
       std::string value = entry.get_string("AdapterRAM", "");
-      if (value.size() > 0)
+      if (!value.empty())
       {
-        int size = atoi(value.c_str()) / 1024 / 1024;
+        int64_t size = base::atoi<int64_t>(value, (int64_t)0) / 1024 / 1024;
         result << tab << "Installed video RAM: " << size << " MB\n";
       }
       else
