@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,8 +16,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-#ifndef _MFORMS_TEXTBOX_H_
-#define _MFORMS_TEXTBOX_H_
+
+#pragma once
 
 #include <mforms/view.h>
 #include <mforms/utilities.h>
@@ -53,17 +53,17 @@ namespace mforms {
 #ifndef SWIG
   struct TextBoxImplPtrs
   {
-    bool (__stdcall *create)(TextBox *self, ScrollBars scroll_bars);
-    void (__stdcall *set_bordered)(TextBox *self, bool flag);
-    void (__stdcall *set_read_only)(TextBox *self, bool flag);
-    void (__stdcall *set_text)(TextBox *self, const std::string &text);
-    std::string (__stdcall *get_text)(TextBox *self);
-    void (__stdcall *append_text)(TextBox *self, const std::string &text, bool scroll_to_end);
-//    void (__stdcall *append_text_with_attributes)(TextBox *self, const std::string &text, const TextAttributes &attr, bool scroll_to_end);
-    void (__stdcall *set_padding)(TextBox *self, int pad); // TODO: not supported on Windows, should be removed.
-    void (__stdcall *set_monospaced)(TextBox *self, bool flag);
-    void (__stdcall *get_selected_range)(TextBox *self, int &start, int &end);
-    void (__stdcall *clear)(TextBox *self);
+    bool (*create)(TextBox *self, ScrollBars scroll_bars);
+    void (*set_bordered)(TextBox *self, bool flag);
+    void (*set_read_only)(TextBox *self, bool flag);
+    void (*set_text)(TextBox *self, const std::string &text);
+    std::string (*get_text)(TextBox *self);
+    void (*append_text)(TextBox *self, const std::string &text, bool scroll_to_end);
+//    void (*append_text_with_attributes)(TextBox *self, const std::string &text, const TextAttributes &attr, bool scroll_to_end);
+    void (*set_padding)(TextBox *self, int pad); // TODO: not supported on Windows, should be removed.
+    void (*set_monospaced)(TextBox *self, bool flag);
+    void (*get_selected_range)(TextBox *self, int &start, int &end);
+    void (*clear)(TextBox *self);
   };
 #endif
 #endif
@@ -72,12 +72,14 @@ namespace mforms {
   class MFORMS_EXPORT TextBox : public View
   {
 #ifdef SWIG
-// workaround for problem with unicode strings and default args in swig
+#if SWIG_VERSION < 0x030003   // starting with SWIG v3.0.3, this is no longer necessary (https://github.com/swig/swig/pull/201)
+//workaround for problem with unicode strings and default args in swig
 %rename(append_text_and_scroll) append_text(const std::string &text, bool scroll_to_end);
 %rename(append_text) append_text(const std::string &text);
 
 %rename(append_text_with_encoding_and_scroll) append_text_with_encoding(const std::string &text, const std::string &encoding, bool scroll_to_end);
 %rename(append_text_with_encoding) append_text_with_encoding(const std::string &text, const std::string &encoding);
+#endif
 #endif
   public:
     TextBox(ScrollBars scroll_bars);
@@ -136,5 +138,3 @@ namespace mforms {
     boost::signals2::signal<bool (KeyCode, ModifierKey, const std::string&)> _key_event_signal;
   };
 };
-
-#endif
