@@ -17,8 +17,7 @@
  * 02110-1301  USA
  */
 
-#ifndef _GRTPP_H_
-#define _GRTPP_H_
+#pragma once
 
 #ifdef __GNUC__
 #include <cxxabi.h>
@@ -33,7 +32,7 @@
 #include <boost/function.hpp>
 #include <libxml/xmlmemory.h>
 #include "base/threading.h"
-
+#include <boost/unordered_map.hpp>
 #include <string>
 
 #ifndef _NODLL_
@@ -552,7 +551,7 @@ namespace grt {
 #else
   #define DEFINE_SSIZE_T_FUNCTIONS
 
-  #ifdef __x86_64__
+  #ifdef __LP64__
   #define DEFINE_INT_FUNCTIONS
   #endif
 
@@ -957,7 +956,7 @@ namespace grt {
 
     IterType iter;
     
-    TypedListConstIterator() : iter(0) {}
+    //TypedListConstIterator() : iter(0) {}
 
     TypedListConstIterator(const TypedListConstIterator &content) : iter(content.iter) {}
     
@@ -2265,7 +2264,8 @@ namespace grt {
     
     Allocator _alloc;
     
-    std::map<std::string,std::string> _attributes;
+    boost::unordered_map<std::string, std::string> _attributes;
+//    std::map<std::string,std::string> _attributes;
     MemberList _members;
     MethodList _methods;
     SignalList _signals;
@@ -2804,7 +2804,6 @@ namespace grt {
   protected:
     friend class MetaClass;
 
-    //std::map<std::string,internal::Object*> _objects;
     std::map<std::string, ObjectRef > _objects_cache;
 
     std::vector<MessageSlot> _message_slot_stack;
@@ -2884,5 +2883,3 @@ namespace grt {
   }
 
 };
-
-#endif
