@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,46 +17,33 @@
  * 02110-1301  USA
  */
 
- 
-#include "stdafx.h"
-
 #include "python_context.h"
 #include "grtpp.h"
 #include "grtpp_util.h"
+#include "grtpp_notifications.h"
 #include "base/string_utilities.h"
 #include "base/file_functions.h"
-
-#ifdef ENABLE_PYTHON_MODULES
+#include "base/util_functions.h"
+#include "base/wb_memory.h"
 
 // python internals
-#ifdef __APPLE__
-#include <Python/node.h>
-#include <Python/grammar.h>
-#include <Python/parsetok.h>
-#include <Python/errcode.h>
-#include <Python/token.h>
-#else
 #include <node.h>
-#include <grammar.h>
-#include <parsetok.h>
+//#include <grammar.h>
+//#include <parsetok.h>
 #include <errcode.h>
 #include <token.h>
-#endif
-
 
 #include "python_grtobject.h"
 #include "python_grtlist.h"
 #include "python_grtdict.h"
 
 #include "base/log.h"
+
 DEFAULT_LOG_DOMAIN("python context")
 
 using namespace grt;
 using namespace base;
 
-const std::string grt::LanguagePython= "python";
-
-
 // used to identify a proper GRT context object as a PyCObject
 static const char *GRTTypeSignature= "GRTCONTEXT";
 // used to identify a GRT value as a PyCObject
@@ -71,12 +58,12 @@ static std::string flatten_class_name(std::string name)
   return name;
 }
 
-PythonContext::PythonContext(GRT *grt, const std::string &module_path)
-: _grt(grt)
-{
-  PyObject *main;
-  static const char *argv[2]= { "/dev/null" , NULL };
+//--------------------------------------------------------------------------------------------------
 
+PythonContextHelper::PythonContextHelper(const std::string &module_path)
+{
+  static const char *argv[2] = { "/dev/null", NULL };
+  std::string wb_pythonpath;
   if (getenv("PYTHON_DEBUG"))
     Py_VerboseFlag = 5;
 
@@ -100,23 +87,47 @@ PythonContext::PythonContext(GRT *grt, const std::string &module_path)
         npath.append(*p);
       }
     }
+
     putenv(g_strdup_printf("PATH=%s", npath.c_str()));
+
+	// This allows to define additional paths for python loadin
+	char *pythonpath = getenv("WB_PYTHONPATH");
+	if (pythonpath)
+		wb_pythonpath = getenv("WB_PYTHONPATH");
   }
 
   putenv(g_strdup_printf("PYTHONHOME=%s\\Python", module_path.c_str()));
-  putenv(g_strdup_printf("PYTHONPATH=%s\\Python;%s\\Python\\DLLs;%s\\Python\\Lib", 
-    module_path.c_str(), module_path.c_str(), module_path.c_str()));
+  putenv(g_strdup_printf("PYTHONPATH=%s\\Python;%s\\Python\\DLLs;%s\\Python\\Lib;%s\\Python\\mysql_libs.zip;%s",
+	  module_path.c_str(), module_path.c_str(), module_path.c_str(), module_path.c_str(), wb_pythonpath.c_str()));
   //putenv("PYTHONHOME=C:\\nowhere");
 #endif
   Py_InitializeEx(0); // skips signal handler init
 
   // Stores the main thread state
   _main_thread_state = PyThreadState_Get();
- 
+
   PySys_SetArgv(1, (char**)argv);
 
   PyEval_InitThreads();
+}
 
+//--------------------------------------------------------------------------------------------------
+
+PythonContextHelper::~PythonContextHelper()
+{
+  PyEval_RestoreThread(_main_thread_state);
+  _main_thread_state = NULL;
+#ifdef _DEBUG
+  Py_Finalize();
+#endif
+}
+
+//--------------------------------------------------------------------------------------------------
+
+PythonContext::PythonContext(GRT *grt, const std::string &module_path)
+: PythonContextHelper(module_path)
+{
+  _grt = grt;
   _grt_list_class= 0;
   _grt_dict_class= 0;
   _grt_object_class= 0;
@@ -124,10 +135,14 @@ PythonContext::PythonContext(GRT *grt, const std::string &module_path)
   
   register_grt_module();
   
-  main= PyImport_AddModule("__main__");
+  PyObject *main = PyImport_AddModule("__main__");
   PyDict_SetItemString(PyModule_GetDict(main),
                        "grt", PyImport_ImportModule("grt"));
-  
+
+  PySys_SetObject((char*)"real_stdout", PySys_GetObject((char*)"stdout"));
+  PySys_SetObject((char*)"real_stderr", PySys_GetObject((char*)"stderr"));
+  PySys_SetObject((char*)"real_stdin", PySys_GetObject((char*)"stdin"));
+
   // make sys.stdout and sys.stderr send stuff to GRT
   PySys_SetObject((char*)"stdout", get_grt_module());
   PySys_SetObject((char*)"stderr", get_grt_module());
@@ -140,19 +155,21 @@ PythonContext::PythonContext(GRT *grt, const std::string &module_path)
   {
     PyObject *path = from_grt(grt::StringRef(base::Logger::log_filename()));
     PyDict_SetItemString(PyModule_GetDict(PyImport_AddModule("grt")), "logpath", path);
-    Py_DECREF(path);
+    Py_XDECREF(path);
   }
 
   PyEval_SaveThread();
+  
+  // listen to all notifications so it can be forwarded to python
+  GRTNotificationCenter::get()->add_grt_observer(this);
+  NotificationCenter::get()->add_observer(this);
 }
 
 
 PythonContext::~PythonContext()
 {
-  PyEval_RestoreThread(_main_thread_state);
-  _main_thread_state = NULL;
-
-  Py_Finalize();
+  GRTNotificationCenter::get()->remove_grt_observer(this);
+  NotificationCenter::get()->remove_observer(this);
 }
 
 
@@ -186,6 +203,33 @@ void PythonContext::add_module_path(const std::string &modpath, bool prepend)
 }
 
 
+void PythonContext::set_user_interrupted(const grt::user_cancelled &exc)
+{
+  PyErr_SetString(PythonContext::get()->_grt_user_interrupt_error, exc.what());
+}
+
+void PythonContext::set_db_access_denied(const grt::db_access_denied &exc)
+{
+  PyErr_SetString(PythonContext::get()->_grt_db_access_denied_error, exc.what());
+}
+
+void PythonContext::set_db_login_error(const grt::db_login_error &exc)
+{
+  PyErr_SetString(PythonContext::get()->_grt_db_login_error, exc.what());
+}
+
+void PythonContext::set_db_error(const grt::db_error &exc)
+{
+  PyObject *arg = Py_BuildValue("(si)", exc.what(), exc.error());
+  PyErr_SetObject(PythonContext::get()->_grt_db_error, arg);
+  Py_DECREF(arg);
+}
+
+void PythonContext::set_db_not_conected(const grt::db_not_connected &exc)
+{
+  PyErr_SetString(PythonContext::get()->_grt_db_not_connected, exc.what());
+}
+
 void PythonContext::set_python_error(const grt::type_error &exc, const std::string &location)
 {
   PyErr_SetString(PyExc_TypeError, (location.empty() ? exc.what() : location + ": " + exc.what()).c_str());
@@ -246,6 +290,88 @@ PythonContext *PythonContext::get_and_check()
 }
 
 
+void PythonContext::set_grt_observer_callable(PyObject *obj)
+{
+  _grt_notification_observer = obj;
+}
+
+
+void PythonContext::handle_grt_notification(const std::string &name, ObjectRef sender, DictRef info)
+{
+  if (_grt_notification_observer)
+  {
+    WillEnterPython lock;
+    
+    PyObject *psender = from_grt(sender);
+    PyObject *pinfo = from_grt(info);
+    PyObject *args = Py_BuildValue("sOO", name.c_str(), psender, pinfo);
+    PyObject *res;
+  
+    if (!(res = PyObject_CallObject(_grt_notification_observer, args)))
+    {
+      log_python_error("Error forwarding GRT notification to Python");
+    }
+    if (res)
+    {
+      Py_DECREF(res);
+    }
+    Py_DECREF(psender);
+    Py_XDECREF(pinfo);
+    Py_DECREF(args);
+  }
+}
+
+
+void PythonContext::handle_notification(const std::string &name, void *sender, base::NotificationInfo &info)
+{
+  if (_grt_notification_observer)
+  {
+    WillEnterPython lock;
+
+    PyObject *infodict = PyDict_New();
+    PyObject *res;
+    
+    for (base::NotificationInfo::iterator i = info.begin(); i != info.end(); ++i)
+    {
+      PyObject *str = PyString_FromString(i->second.c_str());
+      PyDict_SetItemString(infodict, i->first.c_str(), str);
+      Py_DECREF(str);
+    }
+    PyObject *args = Py_BuildValue("sOO", name.c_str(), Py_None, infodict);
+    
+    if (!(res = PyObject_CallObject(_grt_notification_observer, args)))
+    {
+      log_python_error("Error forwarding notification to Python");
+    }
+    if (res)
+    {
+      Py_DECREF(res);
+    }
+    Py_DECREF(args);
+    
+    // apply back changes made to the info dict from python code
+    {
+      PyObject *key, *value;
+      Py_ssize_t pos = 0;
+      
+      while (PyDict_Next(infodict, &pos, &key, &value)) 
+      {
+        std::string s;
+        grt::ValueRef v;
+        v = from_pyobject(value);
+        if (!pystring_to_string(key, s) || !v.is_valid())
+        {
+          log_error("Error in Python notification handler: info dictionary contains an invalid item");
+          continue;
+        }
+        info[s]= v.repr();
+      }
+    }
+    Py_DECREF(infodict);
+  }  
+}
+
+
 static PyObject *grt_print(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
@@ -266,7 +392,7 @@ static PyObject *grt_print(PyObject *self, PyObject *args)
       return NULL;
   }  
   else
-    if (!ctx->pystring_to_string(o, text))
+    if (!ctx->pystring_to_string(o, text, true))
       return NULL;
   
 #ifdef _WIN32
@@ -293,7 +419,7 @@ static PyObject *pylog(base::Logger::LogLevel level, PyObject *args)
   if (!PyArg_ParseTuple(args, "sO", &domain, &o))
     return NULL;
   
-  if (!ctx->pystring_to_string(o, text))
+  if (!ctx->pystring_to_string(o, text, true))
     return NULL;
   
   base::Logger::log(level, domain, "%s", text.c_str());
@@ -353,7 +479,7 @@ static PyObject *grt_send_output(PyObject *self, PyObject *args)
       return NULL;
   }  
   else
-    if (!ctx->pystring_to_string(o, text))
+    if (!ctx->pystring_to_string(o, text, true))
       return NULL;
 
   ctx->get_grt()->send_output(text);
@@ -367,11 +493,12 @@ static PyObject *grt_send_warning(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
   std::string text;
+  std::string detail;
   if (!(ctx= PythonContext::get_and_check()))
     return NULL;
   
-  PyObject *o;
-  if (!PyArg_ParseTuple(args, "O", &o))
+  PyObject *o, *d = NULL;
+  if (!PyArg_ParseTuple(args, "O|O", &o, &d))
   {
     if (PyTuple_Size(args) == 1 && PyTuple_GetItem(args, 0) == Py_None)
     {
@@ -382,10 +509,14 @@ static PyObject *grt_send_warning(PyObject *self, PyObject *args)
       return NULL;
   }  
   else
-    if (!ctx->pystring_to_string(o, text))
+  {
+    if (!ctx->pystring_to_string(o, text, true))
       return NULL;
-
-  ctx->get_grt()->send_warning(text);
+    if (d && !ctx->pystring_to_string(d, detail, true))
+      return NULL;
+  }
+  
+  ctx->get_grt()->send_warning(text, detail);
   
   Py_INCREF(Py_None);
   return Py_None;
@@ -395,11 +526,12 @@ static PyObject *grt_send_info(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
   std::string text;
+  std::string detail;
   if (!(ctx= PythonContext::get_and_check()))
     return NULL;
   
-  PyObject *o;
-  if (!PyArg_ParseTuple(args, "O", &o))
+  PyObject *o, *d = NULL;
+  if (!PyArg_ParseTuple(args, "O|O", &o, &d))
   {
     if (PyTuple_Size(args) == 1 && PyTuple_GetItem(args, 0) == Py_None)
     {
@@ -410,11 +542,15 @@ static PyObject *grt_send_info(PyObject *self, PyObject *args)
       return NULL;
   }  
   else
-    if (!ctx->pystring_to_string(o, text))
+  {
+    if (!ctx->pystring_to_string(o, text, true))
       return NULL;
+    if (d && !ctx->pystring_to_string(d, detail, true))
+      return NULL;
+  }
 
-  ctx->get_grt()->send_info(text);
-  log_debug2("grt.python", text.c_str());
+  ctx->get_grt()->send_info(text, detail);
+//  log_debug2("grt.python", "%s: (%s)", text.c_str(), detail.c_str());
 
   Py_INCREF(Py_None);
   return Py_None;
@@ -424,11 +560,12 @@ static PyObject *grt_send_error(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
   std::string text;
+  std::string detail;
   if (!(ctx= PythonContext::get_and_check()))
     return NULL;
   
-  PyObject *o;
-  if (!PyArg_ParseTuple(args, "O", &o))
+  PyObject *o, *d = NULL;
+  if (!PyArg_ParseTuple(args, "O|O", &o, &d))
   {
     if (PyTuple_Size(args) == 1 && PyTuple_GetItem(args, 0) == Py_None)
     {
@@ -439,10 +576,14 @@ static PyObject *grt_send_error(PyObject *self, PyObject *args)
       return NULL;
   }  
   else
-    if (!ctx->pystring_to_string(o, text))
+  {
+    if (!ctx->pystring_to_string(o, text, true))
       return NULL;
-
-  ctx->get_grt()->send_error(text);
+    if (d && !ctx->pystring_to_string(d, detail, true))
+      return NULL;
+  }
+  
+  ctx->get_grt()->send_error(text, detail);
   
   Py_INCREF(Py_None);
   return Py_None;
@@ -456,15 +597,62 @@ static PyObject *grt_send_progress(PyObject *self, PyObject *args)
     return NULL;
   
   float pct;
-  char *text, *detail= NULL;
-  if (!PyArg_ParseTuple(args, "fs|s", &pct, &text, &detail))
+  PyObject *texto, *detailo= NULL;
+  if (!PyArg_ParseTuple(args, "fO|O", &pct, &texto, &detailo))
     return NULL;
   
-  ctx->get_grt()->send_progress(pct, text ? text : "", detail ? detail : "", NULL);
+  std::string text, detail;
+  if (!ctx->pystring_to_string(texto, text, true))
+    return NULL;
+  if (detailo && !ctx->pystring_to_string(detailo, detail, true))
+    return NULL;
+  ctx->get_grt()->send_progress(pct, text, detail, NULL);
   Py_INCREF(Py_None);
   return Py_None;
 }
 
+static PyObject *grt_begin_progress_step(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+  
+  float pct1, pct2;
+  if (!PyArg_ParseTuple(args, "ff", &pct1, &pct2))
+    return NULL;
+  
+  ctx->get_grt()->begin_progress_step(pct1, pct2);
+
+  Py_RETURN_NONE;
+}
+
+static PyObject *grt_end_progress_step(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+  
+  if (!PyArg_ParseTuple(args, ""))
+    return NULL;
+  
+  ctx->get_grt()->end_progress_step();
+  
+  Py_RETURN_NONE;
+}
+
+static PyObject *grt_reset_progress_steps(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+  
+  if (!PyArg_ParseTuple(args, ""))
+    return NULL;
+  
+  ctx->get_grt()->reset_progress_steps();
+  
+  Py_RETURN_NONE;
+}
 
 static PyObject *grt_get_by_path(PyObject *self, PyObject *args)
 {
@@ -526,23 +714,59 @@ static PyObject *grt_readline(PyObject *self, PyObject *args)
   if (!PyArg_ParseTuple(args, ""))
     return NULL;
   
-  std::string line = ctx->stdin_readline_slot();
-  return Py_BuildValue("s", line.c_str());
+  if (ctx->stdin_readline_slot)
+  {
+    std::string line = ctx->stdin_readline_slot();
+    return Py_BuildValue("s", line.c_str());
+  }
+  PyErr_SetString(PyExc_NotImplementedError, "input()/stdin reading not available");
+  return NULL;
 }
 
+void decrement_int(int *i)
+{
+  (*i)--;
+}
 
-static bool call_handle_output(const std::string &text, PyObject *callable)
+static bool call_handle_message(const grt::Message &msg, void *sender, AutoPyObject callable)
 {
   WillEnterPython lock;
   PyObject *ret;
+  const char *type = NULL;
+  static int handling_message = 0;
   
-  if (!(ret = PyObject_Call(callable, Py_BuildValue("s", text.c_str()), NULL)))
+  if (handling_message > 10)
   {
+    log_warning("Force-breaking apparent recursion of GRT message handlers\n");
+    return false;
+  }
+  handling_message++;
+  base::scope_ptr<int, decrement_int> guard(&handling_message);
+  
+  std::string text = msg.text;
+  switch (msg.type)
+  {
+    case grt::ErrorMsg: type = "ERROR"; break;
+    case grt::WarningMsg: type = "WARNING"; break;
+    case grt::InfoMsg: type = "INFO"; break;
+    case grt::OutputMsg: type = "OUTPUT"; break;
+    case grt::VerboseMsg: type = "VERBOSE"; break;
+    case grt::ProgressMsg: type = "PROGRESS"; text = strfmt("%f:%s", msg.progress, msg.text.c_str()); break;
+    default:
+      type = "???"; 
+      break;
+  }
+  
+  PyObject *args = Py_BuildValue("(sss)", type, text.c_str(), msg.detail.c_str());
+  if (!(ret = PyObject_Call(callable, args, NULL)))
+  {
+    Py_DECREF(args);
     PythonContext::log_python_error("Error calling Python output handler:");
     PyErr_Clear();
     return false;
   }
-  
+  Py_DECREF(args);
+
   if (ret == Py_None || ret == Py_False || PyInt_AsLong(ret) == 0)
   {
     Py_DECREF(ret);
@@ -553,7 +777,7 @@ static bool call_handle_output(const std::string &text, PyObject *callable)
 }
 
 
-static PyObject *grt_push_output_handler(PyObject *self, PyObject *args)
+static PyObject *grt_push_message_handler(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
   if (!(ctx= PythonContext::get_and_check()))
@@ -565,24 +789,98 @@ static PyObject *grt_push_output_handler(PyObject *self, PyObject *args)
   
   if (!PyCallable_Check(o))
     return NULL;
+  
+  ctx->get_grt()->push_message_handler(boost::bind(&call_handle_message, _1, _2, AutoPyObject(o)));
 
-  ctx->push_output_handler(boost::bind(&call_handle_output,_1, AutoPyObject(o)));
-  Py_INCREF(Py_None);
-  return Py_None;
+  return Py_BuildValue("i", ctx->get_grt()->message_handler_count());
 }
 
-static PyObject *grt_pop_output_handler(PyObject *self, PyObject *args)
+
+static PyObject *grt_pop_message_handler(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
   if (!(ctx= PythonContext::get_and_check()))
     return NULL;
-
-  ctx->pop_output_handler();
   
-  Py_INCREF(Py_None);
-  return Py_None;
+  ctx->get_grt()->pop_message_handler();
+  
+  return Py_BuildValue("i", ctx->get_grt()->message_handler_count());
 }
 
+//
+
+static bool call_status_query(AutoPyObject callable)
+{
+  WillEnterPython lock;
+
+  PyObject *ret;
+  PyObject *args = Py_BuildValue("()");
+  if (!(ret = PyObject_Call(callable, args, NULL)))
+  {
+    Py_DECREF(args);
+    PythonContext::log_python_error("Error calling Python status handler:");
+    PyErr_Clear();
+    return false;
+  }
+  Py_DECREF(args);
+  
+  if (ret == Py_None || ret == Py_False || PyInt_AsLong(ret) == 0)
+  {
+    Py_DECREF(ret);
+    return false;
+  }
+  Py_DECREF(ret);  
+  return true;
+}
+
+
+static PyObject *grt_push_status_query_handler(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+  
+  PyObject *o;
+  if (!PyArg_ParseTuple(args, "O", &o))
+    return NULL;
+  
+  if (!PyCallable_Check(o))
+    return NULL;
+  
+  ctx->get_grt()->push_status_query_handler(boost::bind(&call_status_query, AutoPyObject(o)));
+  
+  Py_RETURN_NONE;
+}
+
+
+static PyObject *grt_pop_status_query_handler(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+  
+  ctx->get_grt()->pop_status_query_handler();
+  
+  Py_RETURN_NONE;
+}
+
+static PyObject *grt_query_status(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+  
+  if (args && !PyArg_ParseTuple(args, ""))
+    return NULL;
+  
+  if (ctx->get_grt()->query_status())
+    Py_RETURN_TRUE;
+  else
+    Py_RETURN_FALSE;
+}
+
+//
+
 static PyObject *grt_serialize(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
@@ -653,6 +951,108 @@ static PyObject *grt_unserialize(PyObject *self, PyObject *args)
   return Py_None;
 }
 
+
+static PyObject *grt_set_notification_observer(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+
+  PyObject *object;
+  
+  if (!PyArg_ParseTuple(args, "O", &object))
+    return NULL;
+  
+  if (!PyCallable_Check(object))
+  {
+    PyErr_SetString(PyExc_ValueError, "notification observer argument must be a callable");
+    return NULL;
+  }
+  
+  ctx->set_grt_observer_callable(object);
+  
+  Py_INCREF(Py_None);
+  return Py_None;
+}
+
+
+static PyObject *grt_send_notification(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+  
+  char *name;
+  PyObject *object;
+  PyObject *info;
+  
+  if (!PyArg_ParseTuple(args, "sOO", &name, &object, &info))
+    return NULL;
+  
+  ValueRef infodict = ctx->from_pyobject(info);
+  if (infodict.is_valid() && !DictRef::can_wrap(infodict))
+  {
+    PyErr_SetString(PyExc_ValueError, "notification info must be a dict or None");
+    return NULL;
+  }
+  ValueRef sender = ctx->from_pyobject(object);
+  if (sender.is_valid() && !ObjectRef::can_wrap(sender))
+  {
+    PyErr_SetString(PyExc_ValueError, "notification sender info must be a GRT object");
+    return NULL;
+  }
+
+  try
+  {
+    GRTNotificationCenter::get()->send_grt(name, ObjectRef::cast_from(sender), DictRef::cast_from(infodict));
+  }
+  catch (std::exception &exc)
+  {
+    ctx->set_python_error(exc, "Error sending notification");
+    return NULL;
+  }
+  
+  Py_INCREF(Py_None);
+  return Py_None;
+}
+
+
+static PyObject *(*wrap_pyobject_func)(PyObject *, PyObject*) = NULL;
+static PyObject *(*unwrap_pyobject_func)(PyObject *, PyObject*) = NULL;
+
+static PyObject *grt_wrap_pyobject(PyObject *self, PyObject *args)
+{
+  // since the GRT objects are defined in an outer level, we forward this
+  // to a callback which must be registered beforehand with PythonContext::set_wrap_callback
+   if (wrap_pyobject_func)
+    return wrap_pyobject_func(self, args);
+    
+  PyErr_SetString(PyExc_SystemError, "GRT python support not initialized correctly");
+  return NULL;
+}
+
+
+static PyObject *grt_unwrap_pyobject(PyObject *self, PyObject *args)
+{
+  if (unwrap_pyobject_func)
+    return unwrap_pyobject_func(self, args);
+    
+  PyErr_SetString(PyExc_SystemError, "GRT python support not initialized correctly");
+  return NULL;
+}
+
+
+void PythonContext::set_wrap_pyobject_func(PyObject *(*func)(PyObject *, PyObject*))
+{
+  wrap_pyobject_func = func;
+}
+
+
+void PythonContext::set_unwrap_pyobject_func(PyObject *(*func)(PyObject *, PyObject*))
+{
+  unwrap_pyobject_func = func;
+}
+
 /** Register grt related functionality as a module
 
  Stuff made available in the module include:
@@ -674,6 +1074,12 @@ static PyMethodDef GrtModuleMethods[] = {
 "Write a info message to the GRT shell."},
 {"send_progress",  grt_send_progress, METH_VARARGS,
 "Write a progress message."},
+{"begin_progress_step",  grt_begin_progress_step, METH_VARARGS,
+  "Starts a new step in a larger task where progress is being tracked."},
+{"end_progress_step",  grt_end_progress_step, METH_VARARGS,
+  "Ends a step."},
+{"reset_progress_steps", grt_reset_progress_steps, METH_VARARGS,
+  "Clear the progress steps stack."},
 
 {"log_error", grt_log_error, METH_VARARGS,
 "Logs an error to the log file, in the specified context ex: log_error('myplugin', 'cannot open file')"},
@@ -687,12 +1093,19 @@ static PyMethodDef GrtModuleMethods[] = {
  "Logs a very verbose debug message to the log file, in the specified context ex: log_debug3('myplugin', 'processing file')"},
 {"log_info", grt_log_info, METH_VARARGS,
 "Logs an informational message to the log file, in the specified context ex: log_info('myplugin', 'file opened')"},
-  
-{"push_output_handler", grt_push_output_handler, METH_VARARGS,
-"Pushes a callback of the form function(text) to be called when a plugin outputs text."},
-{"pop_output_handler", grt_push_output_handler, METH_NOARGS,
- "Pops previously pushed handler."},
 
+{"push_message_handler", grt_push_message_handler, METH_VARARGS,
+  "Pushes a callback of the form function((type, text, detail)) to be called when a plugin outputs text. Return value must be True if the message was handled, False if it should be handled by a previously installed handler."},
+{"pop_message_handler", grt_pop_message_handler, METH_NOARGS,
+  "Pops previously pushed handler."},
+  
+{"push_status_query_handler", grt_push_status_query_handler, METH_VARARGS,
+  "Pushes a callback that returns a boolean value. True should be returned for aborting running tasks that call grt.query_status()"},
+{"pop_status_query_handler", grt_pop_status_query_handler, METH_NOARGS,
+  "Pops status_query callback from the stack."},
+{"query_status", grt_query_status, METH_NOARGS,
+  "Queries the current status query handler whether currently running task should be cancelled. If True is returned, the task should be cancelled."},
+  
 {"readline", grt_readline, METH_VARARGS,
 "Waits for a line of text to be input to the scripting shell prompt."},
 
@@ -704,7 +1117,18 @@ static PyMethodDef GrtModuleMethods[] = {
 {"unserialize", grt_unserialize, METH_VARARGS,
   "Unserializes a GRT object from a XML file created by serialize. unserialize(path) -> object"},
 
-  
+{"_set_grt_notification_observer", grt_set_notification_observer, METH_VARARGS,
+  "Sets the GRT notification observer. Internal use."},
+
+{"send_grt_notification", grt_send_notification, METH_VARARGS,
+  "Sends a GRT notification to the application. ex: send_grt_notification('GNMyPluginStarted', sender, infodict)"},
+
+{"fromgrt", grt_unwrap_pyobject, METH_VARARGS,
+  "Extracts the Python object wrapped in a grt_PyObject GRT object (suitable for use with GRT object values)"},
+
+{"togrt", grt_wrap_pyobject, METH_VARARGS,
+  "Wraps a Python object in a grt_PyObject GRT object, which you can then use to reference from GRT objects (such as GRT dicts and lists)"},
+
 {NULL, NULL, 0, NULL}        /* Sentinel */
 };
 
@@ -733,7 +1157,28 @@ void PythonContext::register_grt_module()
   init_grt_list_type();
   init_grt_dict_type();
   init_grt_object_type();
-  
+
+  {
+    _grt_user_interrupt_error = PyErr_NewException((char*)"grt.UserInterrupt", NULL, NULL);
+    PyModule_AddObject(_grt_module, "UserInterrupt", _grt_user_interrupt_error);
+  }
+  {
+    _grt_db_error = PyErr_NewException((char*)"grt.DBError", NULL, NULL);
+    PyModule_AddObject(_grt_module, "DBError", _grt_db_error);
+  }
+  {
+    _grt_db_access_denied_error = PyErr_NewException((char*)"grt.DBAccessDenied", NULL, NULL);
+    PyModule_AddObject(_grt_module, "DBAccessDenied", _grt_db_access_denied_error);
+  }
+  {
+    _grt_db_login_error = PyErr_NewException((char*)"grt.DBLoginError", NULL, NULL);
+    PyModule_AddObject(_grt_module, "DBLoginError", _grt_db_login_error);
+  }
+  {
+    _grt_db_not_connected = PyErr_NewException((char*)"grt.DBNotConnected", NULL, NULL);
+    PyModule_AddObject(_grt_module, "DBNotConnected", _grt_db_not_connected);
+  }
+
   _grt_modules_module = Py_InitModule("grt.modules", NULL);
   if (!_grt_modules_module)
     throw std::runtime_error("Error initializing grt.modules module in Python support");
@@ -879,11 +1324,14 @@ PyObject *PythonContext::from_grt(const ValueRef &value)
     {
       case IntegerType:
       {
+        /*
         long l = *IntegerRef::cast_from(value);
         if ((long)(int)l == l)
           return PyInt_FromLong(l);
         else
           return PyLong_FromLong(l);
+          */
+        return PyInt_FromSsize_t(*IntegerRef::cast_from(value));
       }
 
       case DoubleType:
@@ -893,21 +1341,25 @@ PyObject *PythonContext::from_grt(const ValueRef &value)
       {
         // maybe this should start returning unicode data, but before that all python code
         // should be tested if it can handle the unicode type. For now we just return utf8 strings.
-        //return PyUnicode_EncodeUTF8();
         std::string data= *StringRef::cast_from(value);
+        //return PyUnicode_DecodeUTF8(data.data(), data.size(), NULL);
         return PyString_FromStringAndSize(data.data(), data.size());
       }
       case ListType:
       {
         PyObject *content= PythonContext::internal_cobject_from_value(value);
-        PyObject *r= PyObject_Call(_grt_list_class, Py_BuildValue("(ssO)", "", "", content), NULL);
+        PyObject *args = Py_BuildValue("(ssO)", "", "", content);
+        PyObject *r= PyObject_Call(_grt_list_class, args, NULL);
+        Py_XDECREF(args);
         Py_XDECREF(content);
         return r;
       }
       case DictType:
       {
         PyObject *content= PythonContext::internal_cobject_from_value(value);
-        PyObject *r= PyObject_Call(_grt_dict_class, Py_BuildValue("(ssO)", "", "", content), NULL);
+        PyObject *args = Py_BuildValue("(ssO)", "", "", content);
+        PyObject *r= PyObject_Call(_grt_dict_class, args, NULL);
+        Py_XDECREF(args);
         Py_XDECREF(content);
         return r;
       }
@@ -916,7 +1368,9 @@ PyObject *PythonContext::from_grt(const ValueRef &value)
         std::string class_name= grt::ObjectRef::cast_from(value).class_name();
         PyObject *content= PythonContext::internal_cobject_from_value(value);
         PyObject *theclass= _grt_class_wrappers[class_name];
-        PyObject *r= PyObject_Call(theclass ? theclass : (PyObject*)_grt_object_class, Py_BuildValue("(sO)", "", content), NULL);
+        PyObject *args = Py_BuildValue("(sO)", "", content);
+        PyObject *r= PyObject_Call(theclass ? theclass : (PyObject*)_grt_object_class, args, NULL);
+        Py_XDECREF(args);
         Py_XDECREF(content);
         
         return r;
@@ -929,7 +1383,7 @@ PyObject *PythonContext::from_grt(const ValueRef &value)
   return Py_None;
 }
 
-bool PythonContext::pystring_to_string(PyObject *strobject, std::string &ret_string)
+bool PythonContext::pystring_to_string(PyObject *strobject, std::string &ret_string, bool convert)
 {
   if (PyUnicode_Check(strobject))
   {
@@ -960,6 +1414,18 @@ bool PythonContext::pystring_to_string(PyObject *strobject, std::string &ret_str
       ret_string = "";
     return true;
   }
+  
+  if (convert)
+  {
+    PyObject *str = PyObject_Str(strobject);
+    if (str)
+    {
+      bool ret = pystring_to_string(str, ret_string, false);
+      Py_DECREF(str);
+      return ret;
+    }
+  }
+  
   return false;
 }
 
@@ -1110,7 +1576,7 @@ ValueRef PythonContext::from_pyobject(PyObject *object, const grt::TypeSpec &exp
     {
       if (PyList_Check(object))
       {
-        grt::BaseListRef list(_grt);
+        grt::BaseListRef list(_grt, expected_type.content.type);
         
         for (Py_ssize_t c= PyList_Size(object), i= 0; i < c; i++)
         {
@@ -1127,7 +1593,7 @@ ValueRef PythonContext::from_pyobject(PyObject *object, const grt::TypeSpec &exp
               list.ginsert(from_pyobject(item));
               break;
             default:
-              g_warning("invalid type spec requested");
+              log_warning("invalid type spec requested\n");
               break;
           }
         }
@@ -1162,7 +1628,7 @@ ValueRef PythonContext::from_pyobject(PyObject *object, const grt::TypeSpec &exp
               dict.set(PyString_AsString(key), from_pyobject(value));
               break;
             default:
-              g_warning("invalid type spec requested");
+              log_warning("invalid type spec requested\n");
               break;
           }
         }
@@ -1220,9 +1686,20 @@ int PythonContext::run_buffer(const std::string &buffer, std::string *line_buffe
   PyObject *mainmod;
   PyObject *globals;
   
+
+  std::string old_buff;
   if (line_buffer)
+    old_buff.append(*line_buffer);
+
+  if (line_buffer)
+  {
+    //if previous buff is empty and new command is plain enter key, do nothing
+    if (line_buffer->empty() && buffer[0] == '\n')
+      return 0;
+
     line_buffer->append(buffer);
-  
+  }
+
   WillEnterPython lock;
   
   n= PyParser_SimpleParseStringFlags(line_buffer ? line_buffer->c_str() : buffer.c_str(),
@@ -1233,7 +1710,6 @@ int PythonContext::run_buffer(const std::string &buffer, std::string *line_buffe
   {
     return 0; // continued line
   }
-  
   if (!n && PyErr_Occurred() && PyErr_ExceptionMatches(PyExc_SyntaxError))
   {
     PyObject *excep;
@@ -1242,13 +1718,16 @@ int PythonContext::run_buffer(const std::string &buffer, std::string *line_buffe
     PyObject *trace;
     PyErr_Fetch(&excep, &value, &trace);
     message= PyTuple_GetItem(value, 0);
-    if (strstr(PyString_AsString(message), "unexpected EOF") ||
+    if (strstr(PyString_AsString(message), "expected an indented block") ||
+        strstr(PyString_AsString(message), "unexpected EOF") ||
         strncmp(PyString_AsString(message), "EOF", 3)==0)
     {
       Py_DECREF(excep);
       Py_DECREF(value);
       if (trace)
+      {
         Py_DECREF(trace);
+      }
       PyErr_Clear();
       return 0; // continued line
     }
@@ -1265,6 +1744,7 @@ int PythonContext::run_buffer(const std::string &buffer, std::string *line_buffe
     PyErr_Clear();
     return -1;
   }
+
   PyNode_Free(n);
   PyErr_Clear();
   
@@ -1305,11 +1785,13 @@ int PythonContext::run_buffer(const std::string &buffer, std::string *line_buffe
 // template of a python function to create a wrapper class for a GRT class
 static const char *create_class_template=
 "class %s(grt.Object):\n"
+"  __grtclassname__ = \"%s\"\n"
 "  def __init__(self, classname = None, wrapobj = None):\n"
 "    grt.Object.__init__(self, classname, wrapobj)";
 
 static const char *create_class_template_sub=
 "class %s(%s):\n"
+"  __grtclassname__ = \"%s\"\n"
 "  def __init__(self, classname = '%s', wrapobj = None):\n"
 "    %s.__init__(self, classname, wrapobj)";
 
@@ -1328,12 +1810,14 @@ static void create_class_wrapper(grt::MetaClass *meta, PyObject *locals)
     script= strfmt(create_class_template_sub, 
                    flatten_class_name(meta->name()).c_str(), parname.c_str(), 
                    meta->name().c_str(),
+                   meta->name().c_str(),
                    parname.c_str());
   }
   else
   {
     script= strfmt(create_class_template, 
-                   flatten_class_name(meta->name()).c_str());
+                   flatten_class_name(meta->name()).c_str(),
+                   meta->name().c_str());
   }
 
   if (!PyRun_String(script.c_str(), Py_single_input, locals, locals))
@@ -1367,7 +1851,9 @@ int PythonContext::refresh()
   const std::vector<grt::Module*> &modules(_grt->get_modules());
   for (std::vector<grt::Module*>::const_iterator iter= modules.begin(); iter != modules.end(); ++iter)
   {
-    PyObject *r= PyObject_Call(_grt_module_class, Py_BuildValue("(s)", (*iter)->name().c_str()), NULL);
+    PyObject *arg = Py_BuildValue("(s)", (*iter)->name().c_str());
+    PyObject *r= PyObject_Call(_grt_module_class, arg, NULL);
+    Py_DECREF(arg);
 
     if (!r)
       PythonContext::log_python_error("Error refreshing grt modules");
@@ -1426,5 +1912,3 @@ void PythonContext::run_post_init_script()
   if (PyRun_SimpleString((char*)post_init_script) < 0)
     PythonContext::log_python_error("Error running post-init script:");
 }
-
-#endif // ENABLE_PYTHON_MODULES
