@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -411,7 +411,6 @@ public:
     _editor_host = editor->get_sql_editor()->get_container();
     scoped_connect(editor->get_catalog()->signal_changed(), boost::bind(&MySQLTriggerPanel::catalog_changed, this, _1, _2));
 
-
     set_spacing(15);
     set_padding(4);
     
@@ -620,6 +619,7 @@ public:
   void refresh()
   {
     _refreshing = true;
+    _trigger_list.freeze_refresh();
 
     mforms::TreeNodeRef selected = _trigger_list.get_selected_node();
     int old_selected = 0;
@@ -653,6 +653,7 @@ public:
     _trigger_list.select_node(_trigger_list.node_at_row(old_selected));
 
     _refreshing = false;
+    _trigger_list.thaw_refresh();
   }
   
   //------------------------------------------------------------------------------------------------
@@ -783,44 +784,46 @@ public:
           bool removal_done = false;
           do
           {
-            MySQLToken token = scanner->next_token();
-            if (token.type == ANTLR3_TOKEN_EOF)
+            scanner->next(false);
+            if (scanner->token_type() == ANTLR3_TOKEN_EOF)
               break;
 
-            if (!removal_done && token.type == ordering_token)
+            if (!removal_done && scanner->token_type() == ordering_token)
             {
               // The token we are looking for. Skip this and any whitespace token following it.
               do
               {
-                token = scanner->next_token();
-                if (token.channel == 0 || token.type == ANTLR3_TOKEN_EOF)
+                scanner->next(false);
+                if (scanner->token_channel() == 0 || scanner->token_type() == ANTLR3_TOKEN_EOF)
                   break;
               } while (true);
 
               // See if there's an identifier following the ordering keyword and if so remove that too
               // including the following whitespace).
-              if (scanner->is_identifier(token.type))
+              if (scanner->is_identifier())
               {
                 do
                 {
-                  token = scanner->next_token();
-                  if (token.channel == 0 || token.type == ANTLR3_TOKEN_EOF)
+                  scanner->next(false);
+                  if (scanner->token_channel() == 0 || scanner->token_type() == ANTLR3_TOKEN_EOF)
                     break;
                 } while (true);
               }
 
               removal_done = true;
-              if (token.type == ANTLR3_TOKEN_EOF)
+              if (scanner->token_type() == ANTLR3_TOKEN_EOF)
                 break;
 
               // Add the following token we already scanned or it will get lost.
-              sql += token.text;
+              sql += scanner->token_text();
             }
             else
-              sql += token.text;
+              sql += scanner->token_text();
 
           } while (true);
 
+          delete scanner;
+          
           // Finally remove position information from the trigger object, regardless wether the other trigger actually
           // exists (or is valid) and update the code editor.
           _selected_trigger->ordering("");
@@ -861,7 +864,7 @@ public:
   //------------------------------------------------------------------------------------------------
 
   db_mysql_TriggerRef add_trigger(const std::string &timing, const std::string &event, bool select,
-    const std::string &sql = "")
+    std::string sql = "")
   {
     _editor->freeze_refresh_on_object_change();
     AutoUndoEdit undo(_editor);
@@ -890,11 +893,15 @@ public:
 
       trigger->event(event);
       trigger->timing(timing);
+      sql = base::strfmt("CREATE DEFINER = CURRENT_USER TRIGGER `%s`.`%s` %s %s ON `%s` FOR EACH ROW\nBEGIN\n\nEND\n",
+        _editor->get_schema_name().c_str(), trigger->name().c_str(), timing.c_str(),
+        event.c_str(), _editor->get_name().c_str());
+
+      trigger->sqlDefinition(sql);
     }
     else
-    {
       _editor->_parser_services->parseTrigger(_editor->_parser_context, trigger, sql);
-    }
+
     triggers.insert(trigger);
 
     undo.end(base::strfmt("Add trigger to %s.%s", _editor->get_schema_name().c_str(), _editor->get_name().c_str()));
@@ -945,10 +952,12 @@ public:
     if (_refreshing)
       return;
 
+    if (_code_editor->is_dirty())
+          code_edited();
+
     update_ui();
 
-    if (_code_editor->is_dirty())
-      code_edited();
+
   }
 
   //------------------------------------------------------------------------------------------------
@@ -976,19 +985,7 @@ public:
       _selected_trigger = trigger;
 
       if (trigger.is_valid())
-      {
-        std::string sql;
-        if (trigger->sqlDefinition().empty())
-        {
-          sql = base::strfmt("CREATE DEFINER = CURRENT_USER TRIGGER `%s`.`%s` %s %s ON `%s` FOR EACH ROW\n    ",
-            _editor->get_schema_name().c_str(), trigger->name().c_str(), trigger->timing().c_str(),
-            trigger->event().c_str(), _editor->get_name().c_str());
-        }
-        else
-          sql = trigger->sqlDefinition();
-
-        _editor->get_sql_editor()->sql(sql.c_str());
-      }
+        _editor->get_sql_editor()->sql(trigger->sqlDefinition().c_str());
     }
 
     _editor_host->show(trigger.is_valid());
@@ -1081,9 +1078,10 @@ public:
   {
     mforms::TreeNodeRef new_node = new_parent->insert_child(index);
     new_node->set_string(0, node->get_string(0));
-    new_node->set_tag(node->get_tag());
+    std::string tag = node->get_tag();
     new_node->set_data(node->get_data());
     node->remove_from_parent();
+    new_node->set_tag(tag);
     return new_node;
   }
 
@@ -1373,31 +1371,31 @@ public:
     bool replace_done = false;
     do
     {
-      MySQLToken token = scanner->next_token();
-      if (token.type == ANTLR3_TOKEN_EOF)
+      scanner->next(false);
+      if (scanner->token_type() == ANTLR3_TOKEN_EOF)
         break;
 
-      if (!replace_done && token.type == timing_token)
+      if (!replace_done && scanner->token_type() == timing_token)
       {
         // The token we are looking for. Replace the timing and see if there's an event token too.
         sql += timing;
         do
         { // Add any following hidden tokens (whitespace/comment).
-          token = scanner->next_token();
-          if (token.channel == 0 || token.type == ANTLR3_TOKEN_EOF)
+          scanner->next(false);
+          if (scanner->token_channel() == 0 || scanner->token_type() == ANTLR3_TOKEN_EOF)
             break;
-          sql += token.text;
+          sql += scanner->token_text();
         } while (true);
 
-        if (token.type == event_token)
+        if (scanner->token_type() == event_token)
           sql += event;
 
         replace_done = true;
-        if (token.type == ANTLR3_TOKEN_EOF)
+        if (scanner->token_type() == ANTLR3_TOKEN_EOF)
           break;
       }
       else
-        sql += token.text;
+        sql += scanner->token_text();
 
     } while (true);
 
@@ -1456,6 +1454,14 @@ MySQLTableEditorBE::~MySQLTableEditorBE()
 
 //--------------------------------------------------------------------------------------------------
 
+void MySQLTableEditorBE::refresh_live_object()
+{
+  TableEditorBE::refresh_live_object();
+  load_trigger_sql();
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void MySQLTableEditorBE::commit_changes()
 {
   _trigger_panel->code_edited();
@@ -1596,7 +1602,7 @@ void MySQLTableEditorBE::set_table_option_by_name(const std::string& name, const
     {
       if (_table.get_metaclass()->get_member_type(table_options[i].object_field).base.type == grt::IntegerType)
       {
-        int ivalue= atoi(value.c_str());
+        int ivalue= base::atoi<int>(value, 0);
 
         if (ivalue != *grt::IntegerRef::cast_from(_table.get_member(table_options[i].object_field)))
         {
@@ -1707,6 +1713,7 @@ std::string MySQLTableEditorBE::get_table_option_by_name(const std::string& name
 
 /**
  * Loads the current trigger sql text into the editor control and marks that as not dirty.
+ * In addition the trigger UI is refreshed so that the trigger tree contains update trigger references.
  */
 void MySQLTableEditorBE::load_trigger_sql()
 {
@@ -1730,7 +1737,7 @@ bool MySQLTableEditorBE::can_close()
 
 bool MySQLTableEditorBE::set_partition_type(const std::string &type)
 {
-  if (type.compare(*_table->partitionType())!=0)
+  if (!type.empty() && type.compare(*_table->partitionType())!=0)
   {
     if (type == "RANGE" || type == "LIST")
     {
@@ -1762,6 +1769,21 @@ bool MySQLTableEditorBE::set_partition_type(const std::string &type)
       return true;
     }
   }
+  else if (type.empty())
+  {
+    AutoUndoEdit undo(this);
+    _table->partitionType(type);
+    _table->partitionCount(0);
+    _table->partitionExpression("");
+    _table->subpartitionCount(0);
+    _table->subpartitionExpression("");
+    _table->subpartitionType("");
+    if (get_explicit_partitions())
+      reset_partition_definitions((int)_table->partitionCount(), 0);
+    update_change_date();
+    undo.end(strfmt(_("Disable Partitioning for '%s'"), get_name().c_str()));
+    return true;
+  }
   return false;
 }
 
@@ -2291,10 +2313,10 @@ bool MySQLTableIndexListBE::set_field(const NodeId &node, ColumnId column, const
     }
     return true;
   case RowBlockSize:
-    if (atoi(value.c_str()) != *index->keyBlockSize())
+    if (base::atoi<int>(value, 0) != *index->keyBlockSize())
     {
       AutoUndoEdit undo(_owner, index, "keyBlockSize");
-      index->keyBlockSize(atoi(value.c_str()));
+      index->keyBlockSize(base::atoi<int>(value, 0));
       undo.end(strfmt(_("Change Key Block Size of Index '%s.%s'"), _owner->get_name().c_str(), index->name().c_str()));
     }
     return true;
