@@ -1,26 +1,23 @@
 layout = [ ( 'General',
     [ ( 'Features',
-        [ 'federated',
-          'enable-locking',
+        [ 'enable-locking',
           'enable-pstack',
           'event-scheduler',
+          'federated',
           'isam',
-          'partition',
           'plugin',
           'plugin-load',
           'plugin-load-add',
           'profiling_history_size',
-          'skip-event-scheduler',
           'skip-partition']),
       ('Transactions', ['autocommit']),
       ( 'Memory usage',
-        [ 'host_cache_size',
-          'join_buffer_size',
+        [ 'join_buffer_size',
           'join_cache_level',
           'large-pages',
           'query_alloc_block_size',
           'sort_buffer_size']),
-      ('Informational', ['server-id', 'server-id-bits']),
+      ('Informational', ['server-id']),
       ('Directories', ['basedir', 'datadir', 'plugin_dir', 'tmpdir']),
       ( 'System',
         [ 'core-file',
@@ -29,33 +26,44 @@ layout = [ ( 'General',
           'ignore-db-dir',
           'lower_case_table_names',
           'pid-file',
-          'skip-external-locking',
           'skip-host-cache',
           'skip-locking',
+          'skip-safemalloc',
           'skip-stack-trace',
           'skip-symlink',
+          'skip_external_locking',
           'symbolic-links',
           'sysdate-is-now',
           'use-symbolic-links',
-          'user',
-          'skip-safemalloc',
-          'lower_case_file_system']),
+          'user']),
       ( 'General',
-        [ 'auto_increment_increment',
-          'auto_increment_offset',
-          'big-tables',
+        [ 'big-tables',
           'default-storage-engine',
-          'default_tmp_storage_engine',
           'default-table-type',
+          'default_tmp_storage_engine',
           'init-file',
           'lock_wait_timeout',
           'max_long_data_size',
-          'stored_program_cache',
+          'max_statement_time',
           'old',
-          'one-thread']),
-      ( 'Iternational',
-        [ 'character-set-client-handshake',
-          'character-set-filesystem',
+          'one-thread',
+          'session_track_gtids',
+          'session_track_schema',
+          'session_track_state_change',
+          'session_track_system_variables',
+          'stored_program_cache',
+          'super-large-pages']),
+      ( 'Firewall',
+        [ 'mysql_firewall_max_query_size',
+          'mysql_firewall_mode',
+          'mysql_firewall_trace']),
+      ( 'SQL',
+        [ 'ansi',
+          'explicit_defaults_for_timestamp',
+          'sql-bin-update-same',
+          'sql-mode']),
+      ( 'International',
+        [ 'character-set-filesystem',
           'character-set-server',
           'character-sets-dir',
           'collation-server',
@@ -66,16 +74,9 @@ layout = [ ( 'General',
           'language',
           'lc-messages',
           'lc-messages-dir',
-          'skip-character-set-client-handshake',
-          'time_zone']),
-      ( 'SQL',
-        [ 'ansi',
-          'explicit_defaults_for_timestamp',
-          'sql-bin-update-same',
-          'sql-mode']),
+          'skip-character-set-client-handshake']),
       ( 'Backup',
-        [ 'backup_elevation',
-          'backup_history_log',
+        [ 'backup_history_log',
           'backup_history_log_file',
           'backup_progress_log',
           'backup_progress_log_file',
@@ -83,7 +84,18 @@ layout = [ ( 'General',
           'log-backup-output',
           'mysql-backup'])]),
   ( 'Logging',
-    [ ('Audit', ['audit-log']),
+    [ ( 'Audit',
+        [ 'audit-log',
+          'audit_log_buffer_size',
+          'audit_log_connection_policy',
+          'audit_log_exclude_accounts',
+          'audit_log_file',
+          'audit_log_format',
+          'audit_log_include_accounts',
+          'audit_log_policy',
+          'audit_log_rotate_on_size',
+          'audit_log_statement_policy',
+          'audit_log_strategy']),
       ( 'Slow query log options',
         [ 'log-slow-admin-statements',
           'log-slow-queries',
@@ -107,23 +119,34 @@ layout = [ ( 'General',
           'log-queries-not-using-indexes',
           'log-raw',
           'log-short-format',
-          'log-slave-updates',
+          'log_error_verbosity',
+          'log_slave_updates',
+          'log_syslog',
+          'log_syslog_facility',
+          'log_syslog_include_pid',
+          'log_syslog_tag',
+          'log_timestamps',
           'slow-query-log']),
       ( 'Binlog Options',
         [ 'binlog-checksum',
           'binlog-do-db',
+          'binlog-format',
           'binlog-ignore-db',
           'binlog-row-event-max-size',
-          'binlog_cache_size',
-          'binlog_stmt_cache_size',
-          'binlog-format',
           'binlog-rows-query-log-events',
+          'binlog_cache_size',
           'binlog_direct_non_transactional_updates',
+          'binlog_error_action',
+          'binlog_group_commit_sync_delay',
+          'binlog_group_commit_sync_no_delay_count',
+          'binlog_gtid_simple_recovery',
+          'binlog_row_image',
+          'binlog_stmt_cache_size',
+          'binlogging_impossible_mode',
           'log-bin',
           'log-bin-index',
           'log-bin-trust-function-creators',
           'log-bin-trust-routine-creators',
-          'log-bin-use-v1-row-events',
           'log_bin_use_v1_row_events',
           'max-binlog-dump-events',
           'max_binlog_cache_size',
@@ -143,10 +166,10 @@ layout = [ ( 'General',
           'innodb_buffer_pool_dump_at_shutdown',
           'innodb_buffer_pool_dump_now',
           'innodb_buffer_pool_filename',
+          'innodb_buffer_pool_instances',
           'innodb_buffer_pool_load_abort',
           'innodb_buffer_pool_load_at_startup',
           'innodb_buffer_pool_load_now',
-          'innodb_buffer_pool_instances',
           'innodb_buffer_pool_size']),
       ( 'Logfiles',
         [ 'innodb_flush_log_at_trx_commit',
@@ -159,6 +182,7 @@ layout = [ ( 'General',
           'innodb_log_file_size',
           'innodb_log_files_in_group',
           'innodb_log_group_home_dir',
+          'innodb_log_write_ahead_size',
           'innodb_mirrored_log_groups',
           'innodb_online_alter_log_max_size',
           'innodb_undo_directory',
@@ -168,22 +192,27 @@ layout = [ ( 'General',
         [ 'innodb_additional_mem_pool_size',
           'innodb_buffer_pool_instances',
           'innodb_buffer_pool_size',
+          'innodb_page_cleaners',
           'innodb_page_size',
           'innodb_sort_buffer_size']),
       ( 'Fulltext search',
         [ 'innodb_ft_aux_table',
+          'innodb_ft_cache_size',
+          'innodb_ft_enable_diag_print',
           'innodb_ft_enable_stopword',
-          'innodb_ft_num_word_optimize',
-          'innodb_ft_server_stopword_table',
-          'innodb_ft_user_stopword_table',
           'innodb_ft_max_token_size',
           'innodb_ft_min_token_size',
-          'innodb_ft_cache_size',
+          'innodb_ft_num_word_optimize',
+          'innodb_ft_result_cache_limit',
+          'innodb_ft_server_stopword_table',
           'innodb_ft_sort_pll_degree',
+          'innodb_ft_total_cache_size',
+          'innodb_ft_user_stopword_table',
           'innodb_optimize_fulltext_only']),
       ( 'General',
         [ 'ignore-builtin-innodb',
-          'innodb',
+          'innodb-safe-binlog',
+          'innodb-status-file',
           'innodb_adaptive_flushing',
           'innodb_adaptive_flushing_lwm',
           'innodb_adaptive_hash_index',
@@ -195,6 +224,7 @@ layout = [ ( 'General',
           'innodb_api_trx_level',
           'innodb_autoextend_increment',
           'innodb_autoinc_lock_mode',
+          'innodb_buffer_pool_dump_pct',
           'innodb_change_buffer_max_size',
           'innodb_change_buffering',
           'innodb_checksum_algorithm',
@@ -205,10 +235,13 @@ layout = [ ( 'General',
           'innodb_compression_level',
           'innodb_compression_pad_pct_max',
           'innodb_concurrency_tickets',
+          'innodb_create_intrinsic',
+          'innodb_disable_sort_file_cache',
           'innodb_doublewrite',
           'innodb_extra_dirty_writes',
           'innodb_file_io_threads',
           'innodb_file_per_table',
+          'innodb_fill_factor',
           'innodb_flushing_avg_loops',
           'innodb_force_load_corrupted',
           'innodb_force_recovery',
@@ -223,6 +256,7 @@ layout = [ ( 'General',
           'innodb_max_merged_io',
           'innodb_max_purge_lag',
           'innodb_max_purge_lag_delay',
+          'innodb_max_undo_log_size',
           'innodb_monitor_disable',
           'innodb_monitor_enable',
           'innodb_monitor_reset',
@@ -231,6 +265,7 @@ layout = [ ( 'General',
           'innodb_old_blocks_time',
           'innodb_online_alter_log_max_size',
           'innodb_open_files',
+          'innodb_optimize_fulltext_only',
           'innodb_print_all_deadlocks',
           'innodb_purge_batch_size',
           'innodb_purge_threads',
@@ -241,22 +276,23 @@ layout = [ ( 'General',
           'innodb_replication_delay',
           'innodb_rollback_on_timeout',
           'innodb_rollback_segments',
-          'innodb-safe-binlog',
+          'innodb_sort_buffer_size',
+          'innodb_spin_wait_delay',
           'innodb_stats_auto_recalc',
           'innodb_stats_method',
           'innodb_stats_on_metadata',
           'innodb_stats_persistent',
           'innodb_stats_persistent_sample_pages',
+          'innodb_stats_sample_pages',
           'innodb_stats_transient_sample_pages',
-          'innodb-status-file',
+          'innodb_status_output',
+          'innodb_status_output_locks',
+          'innodb_strict_mode',
           'innodb_support_xa',
           'innodb_sync_array_size',
           'innodb_sync_spin_loops',
-          'innodb_sort_buffer_size',
-          'innodb_spin_wait_delay',
-          'innodb_stats_sample_pages',
-          'innodb_strict_mode',
           'innodb_table_locks',
+          'innodb_temp_data_file_path',
           'innodb_thread_concurrency',
           'innodb_thread_concurrency_timer_based',
           'innodb_thread_sleep_delay',
@@ -268,11 +304,9 @@ layout = [ ( 'General',
           'innodb_use_sys_malloc',
           'innodb_write_io_threads',
           'timed_mutexes',
-          'innodb_optimize_fulltext_only',
           'skip-innodb'])]),
   ( 'Networking',
-    [ ( 'Data / Memory size',
-        ['host_cache_size', 'max_allowed_packet', 'net_buffer_length']),
+    [ ('Data / Memory size', ['max_allowed_packet', 'net_buffer_length']),
       ( 'SSL',
         [ 'ssl',
           'ssl-ca',
@@ -281,9 +315,7 @@ layout = [ ( 'General',
           'ssl-cipher',
           'ssl-crl',
           'ssl-crlpath',
-          'ssl-key',
-          'ssl-verify-server-cert',
-          'skip-ssl']),
+          'ssl-key']),
       ( 'Timeout Settings',
         [ 'connect_timeout',
           'interactive_timeout',
@@ -296,14 +328,13 @@ layout = [ ( 'General',
           'max_user_connections',
           'net_retry_count']),
       ( 'General',
-        [ 'back_log',
-          'bind-address',
+        [ 'bind-address',
+          'enable-named-pipe',
           'port',
           'port-open-timeout',
           'skip-name-resolve',
           'skip-networking',
-          'socket',
-          'enable-named-pipe'])]),
+          'socket'])]),
   ( 'Advanced',
     [ ( 'Transactions',
         [ 'completion_type',
@@ -312,25 +343,19 @@ layout = [ ( 'General',
           'transaction_alloc_block_size',
           'transaction_prealloc_size']),
       ( 'Deprecated',
-        [ 'skip-isam',
-          'old-protocol',
-          'old-alter-table',
-          'set-variable',
-          'skip-new',
-          'sync-bdb-logs',
-          'warnings']),
+        ['old-alter-table', 'old-protocol', 'set-variable', 'warnings']),
       ( 'Thread specific settings',
         [ 'slow_launch_time',
           'thread_cache_size',
           'thread_concurrency',
           'thread_handling',
-          'thread_pool_size',
-          'thread_stack',
           'thread_pool_algorithm',
           'thread_pool_high_priority_connection',
           'thread_pool_max_unused_threads',
           'thread_pool_prio_kickup_timer',
-          'thread_pool_stall_limit']),
+          'thread_pool_size',
+          'thread_pool_stall_limit',
+          'thread_stack']),
       ( 'Various',
         [ 'bulk_insert_buffer_size',
           'group_concat_max_len',
@@ -359,14 +384,15 @@ layout = [ ( 'General',
       ( 'General',
         [ 'bootstrap',
           'chroot',
+          'debug',
           'debug-sync-timeout',
           'div_precision_increment',
-          'eq_range_index_dive_limit',
           'exit-info',
           'flush',
           'flush_time',
           'init_connect',
           'init_slave',
+          'max_digest_length',
           'max_error_count',
           'max_prepared_stmt_count',
           'max_sp_recursion_depth',
@@ -381,49 +407,50 @@ layout = [ ( 'General',
           'skip-concurrent-insert',
           'skip-thread-priority',
           'slow-start-timeout',
+          'standalone',
           'sync_frm',
-          'updatable_views_with_limit',
-          'debug',
-          'standalone'])]),
+          'updatable_views_with_limit'])]),
   ( 'Other',
     [ ( 'Storage Engines',
-        [ 'skip-bdb',
-          'skip-merge',
-          'skip-sync-bdb-logs',
-          'skip-falcon',
-          'skip-ndbcluster']),
+        ['skip-bdb', 'skip-falcon', 'skip-merge', 'skip-sync-bdb-logs']),
       ( 'NDB',
-        [ 'ndb_autoincrement_prefetch_sz',
-          'ndb-batch-size',
+        [ 'ndb-batch-size',
           'ndb-blob-read-batch-bytes',
           'ndb-blob-write-batch-bytes',
-          'ndb_cache_check_time',
           'ndb-cluster-connection-pool',
           'ndb-connectstring',
-          'ndb_extra_logging',
-          'ndb_force_send',
-          'ndb-log-apply-status',
           'ndb-log-transaction-id',
-          'ndb_log_updated_only',
           'ndb-log-update-as-write',
-          'ndb-log-empty-epochs',
-          'ndb_log_empty_epochs',
-          'ndb-nodeid',
           'ndb-mgmd-host',
+          'ndb-nodeid',
+          'ndb-recv-thread-activation-threshold',
+          'ndb-recv-thread-cpu-mask',
           'ndb-wait-connected',
           'ndb-wait-setup',
-          'ndbcluster',
-          'ndb-deferred-constraints',
+          'ndb_autoincrement_prefetch_sz',
+          'ndb_cache_check_time',
           'ndb_deferred_constraints',
-          'ndb-distribution',
           'ndb_distribution',
+          'ndb_eventbuffer_free_percent',
+          'ndb_eventbuffer_max_alloc',
+          'ndb_extra_logging',
+          'ndb_force_send',
           'ndb_index_stat_cache_entries',
           'ndb_index_stat_enable',
+          'ndb_index_stat_option',
           'ndb_index_stat_update_freq',
+          'ndb_log_apply_status',
+          'ndb_log_empty_epochs',
+          'ndb_log_exclusive_reads',
+          'ndb_log_orig',
+          'ndb_log_updated_only',
           'ndb_optimized_node_selection',
           'ndb_report_thresh_binlog_epoch_slip',
           'ndb_report_thresh_binlog_mem_usage',
-          'ndb_use_transactions']),
+          'ndb_show_foreign_key_mock_tables',
+          'ndb_slave_conflict_role',
+          'ndb_use_transactions',
+          'ndbcluster']),
       ( 'Memcached',
         [ 'daemon_memcached_enable_binlog',
           'daemon_memcached_engine_lib_name',
@@ -436,6 +463,7 @@ layout = [ ( 'General',
         [ 'allow-suspicious-udfs',
           'authentication_windows_log_level',
           'authentication_windows_use_principal_name',
+          'block_encryption_mode',
           'des-key-file',
           'read_only',
           'safe-mode',
@@ -448,36 +476,34 @@ layout = [ ( 'General',
           'skip-grant-tables',
           'skip-show-database']),
       ( 'Validate password',
-        [ 'validate-password',
+        [ 'backup_elevation',
           'validate_password_dictionary_file',
           'validate_password_length',
           'validate_password_mixed_case_count',
           'validate_password_number_count',
-          'validate_password_policy_number',
+          'validate_password_policy',
           'validate_password_special_char_count']),
-      ('Authentication', ['default-authentication-plugin', 'old_passwords'])]),
+      ( 'Authentication',
+        [ 'default_authentication_plugin',
+          'default_password_lifetime',
+          'disconnect_on_expired_password',
+          'old_passwords'])]),
   ( 'Replication',
     [ ( 'Relay Log',
         [ 'max_relay_log_size',
           'relay-log',
-          'relay-log-index',
-          'relay-log-info-file',
-          'relay_log_info_file',
           'relay-log-info-repository',
           'relay_log_index',
+          'relay_log_info_file',
           'relay_log_purge',
-          'relay-log-recovery',
           'relay_log_recovery',
           'relay_log_space_limit',
           'sync_relay_log',
           'sync_relay_log_info']),
       ( 'Master',
-        [ 'auto_increment_increment',
-          'auto_increment_offset',
-          'master-connect-retry',
+        [ 'master-connect-retry',
           'master-host',
           'master-info-file',
-          'master-info-repository',
           'master-password',
           'master-port',
           'master-retry-count',
@@ -489,31 +515,33 @@ layout = [ ( 'General',
           'master-ssl-key',
           'master-user',
           'master-verify-checksum',
+          'master_info_repository',
           'sync_master_info']),
       ( 'Slave',
-        [ 'slave_exec_mode',
-          'abort-slave-event-count',
+        [ 'abort-slave-event-count',
           'disconnect-slave-event-count',
           'init_slave',
+          'rpl_stop_slave_timeout',
           'show-slave-auth-info',
-          'slave_allow_batching',
           'slave-load-tmpdir',
-          'slave-skip-errors',
-          'slave-checkpoint-group',
-          'slave_checkpoint_group',
-          'slave-checkpoint-period',
-          'slave_checkpoint_period',
-          'slave_compressed_protocol',
           'slave-max-allowed-packet',
           'slave-net-timeout',
+          'slave-parallel-type',
           'slave-parallel-workers',
           'slave-rows-search-algorithms',
+          'slave-skip-errors',
           'slave-sql-verify-checksum',
+          'slave_allow_batching',
+          'slave_checkpoint_group',
+          'slave_checkpoint_period',
+          'slave_compressed_protocol',
+          'slave_exec_mode',
           'slave_transaction_retries',
           'slave_type_conversions']),
       ( 'General',
-        [ 'disable-gtid-unsafe-statements',
-          'disable_gtid_unsafe_statements',
+        [ 'disable_gtid_unsafe_statements',
+          'enforce_gtid_consistency',
+          'executed-gtids-compression-period',
           'gtid-mode',
           'init-rpl-role',
           'replicate-do-db',
@@ -532,12 +560,12 @@ layout = [ ( 'General',
   ( 'MyISAM',
     [ ( 'Advanced Settings',
         [ 'myisam-block-size',
+          'myisam-recover',
+          'myisam-recover-options',
           'myisam_data_pointer_size',
           'myisam_max_extra_sort_file_size',
           'myisam_max_sort_file_size',
           'myisam_mmap_size',
-          'myisam-recover',
-          'myisam-recover-options',
           'myisam_repair_threads',
           'myisam_sort_buffer_size',
           'myisam_stats_method',
@@ -567,41 +595,64 @@ layout = [ ( 'General',
           'optimizer_search_depth',
           'optimizer_switch']),
       ( 'Performance Schema',
-        [ 'performance_schema',
-          'performance_schema_consumer_xxx',
+        [ 'performance-schema-consumer-events-stages-current',
+          'performance-schema-consumer-events-stages-history',
+          'performance-schema-consumer-events-stages-history-long',
+          'performance-schema-consumer-events-statements-current',
+          'performance-schema-consumer-events-statements-history',
+          'performance-schema-consumer-events-statements-history-long',
+          'performance-schema-consumer-events-transactions-current',
+          'performance-schema-consumer-events-transactions-history',
+          'performance-schema-consumer-events-transactions-history-long',
+          'performance-schema-consumer-events-waits-current',
+          'performance-schema-consumer-events-waits-history',
+          'performance-schema-consumer-events-waits-history-long',
+          'performance-schema-consumer-global-instrumentation',
+          'performance-schema-consumer-statements-digest',
+          'performance-schema-consumer-thread-instrumentation',
+          'performance-schema-instrument',
+          'performance_schema',
+          'performance_schema_accounts_size',
           'performance_schema_digests_size',
           'performance_schema_events_stages_history_long_size',
           'performance_schema_events_stages_history_size',
           'performance_schema_events_statements_history_long_size',
           'performance_schema_events_statements_history_size',
+          'performance_schema_events_transactions_history_long_size',
+          'performance_schema_events_transactions_history_size',
           'performance_schema_events_waits_history_long_size',
           'performance_schema_events_waits_history_size',
           'performance_schema_hosts_size',
-          'performance_schema_instrument',
           'performance_schema_max_cond_classes',
           'performance_schema_max_cond_instances',
           'performance_schema_max_file_classes',
           'performance_schema_max_file_handles',
           'performance_schema_max_file_instances',
+          'performance_schema_max_index_stat',
+          'performance_schema_max_memory_classes',
+          'performance_schema_max_metadata_locks',
           'performance_schema_max_mutex_classes',
           'performance_schema_max_mutex_instances',
+          'performance_schema_max_prepared_statements_instances',
+          'performance_schema_max_program_instances',
           'performance_schema_max_rwlock_classes',
           'performance_schema_max_rwlock_instances',
-          'performance_schema_max_stage_classes',
           'performance_schema_max_socket_classes',
           'performance_schema_max_socket_instances',
+          'performance_schema_max_sql_text_length',
+          'performance_schema_max_stage_classes',
           'performance_schema_max_statement_classes',
+          'performance_schema_max_statement_stack',
           'performance_schema_max_table_handles',
           'performance_schema_max_table_instances',
+          'performance_schema_max_table_lock_stat',
           'performance_schema_max_thread_classes',
           'performance_schema_max_thread_instances',
+          'performance_schema_session_connect_attrs_size',
           'performance_schema_setup_actors_size',
           'performance_schema_setup_objects_size',
           'performance_schema_users_size']),
       ( 'General',
         [ 'delay-key-write',
           'low-priority-updates',
-          'profiling_history_size',
-          'table_definition_cache',
-          'table_open_cache',
-          'table_open_cache_instances'])])]
+          'profiling_history_size'])])]
