@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -30,19 +30,34 @@ except IOError, err:
 import os
 import errno
 import stat
-#import mforms
+import mforms
 import time
 import traceback
 
 from wb_common import PermissionDeniedError, InvalidPasswordError, OperationCancelledError, Users
 from workbench.utils import server_version_str2tuple
-from wb_common import CmdOptions, CmdOutput
+from wb_common import CmdOptions, CmdOutput, SSHFingerprintNewError, format_bad_host_exception
 
 from workbench.log import log_info, log_warning, log_error, log_debug, log_debug2, log_debug3
 
 try:
     import paramiko
     import socket
+    
+    class StoreIfConfirmedPolicy(paramiko.MissingHostKeyPolicy):
+        def missing_host_key(self, client, hostname, key):
+            raise SSHFingerprintNewError("Key mismatched", client, hostname, key)
+        
+    # paramiko 1.6 didn't have this class
+    if hasattr(paramiko, "WarningPolicy"):
+        WarningPolicy = paramiko.WarningPolicy
+    else:
+        class WarningPolicy(paramiko.MissingHostKeyPolicy):
+            def missing_host_key(self, client, hostname, key):
+                import binascii
+                log_warning('WARNING: Unknown %s host key for %s: %s\n' % (key.get_name(), hostname, binascii.hexlify(key.get_fingerprint())))
+
+    
 except:
     traceback.print_exc()
     # temporary workaround
@@ -73,20 +88,37 @@ if paramiko and server_version_str2tuple(paramiko.__version__) >= (1, 7, 4):
     import threading
     OPEN_CHANNEL_TIMEOUT = 15
 
-    def wba_open_channel(self, kind, dest_addr=None, src_addr=None, timeout = None):
+    def wba_open_channel(self, kind, dest_addr=None, src_addr=None, timeout = None, window_size=None, max_packet_size=None):
         chan = None
         if not self.active:
             # don't bother trying to allocate a channel
             return None
         self.lock.acquire()
         try:
+            
             chanid = self._next_channel()
             m = Message()
             m.add_byte(chr(MSG_CHANNEL_OPEN))
             m.add_string(kind)
             m.add_int(chanid)
-            m.add_int(self.window_size)
-            m.add_int(self.max_packet_size)
+            local_window_size = None
+            local_max_packet_size = None
+            if (server_version_str2tuple(paramiko.__version__) < (1, 15, 0)):
+                local_window_size = self.window_size
+                local_max_packet_size = self.max_packet_size
+            else:
+                if window_size is not None:
+                    local_window_size = self._sanitize_window_size(window_size)
+                else:
+                    local_window_size = self.default_window_size
+                
+                if max_packet_size is not None:
+                    local_max_packet_size = self._sanitize_packet_size(max_packet_size)
+                else:
+                    local_max_packet_size = self.default_max_packet_size
+
+            m.add_int(local_window_size)
+            m.add_int(local_max_packet_size)
             if (kind == 'forwarded-tcpip') or (kind == 'direct-tcpip'):
                 m.add_string(dest_addr[0])
                 m.add_int(dest_addr[1])
@@ -100,7 +132,7 @@ if paramiko and server_version_str2tuple(paramiko.__version__) >= (1, 7, 4):
             self.channel_events[chanid] = event = threading.Event()
             self.channels_seen[chanid] = True
             chan._set_transport(self)
-            chan._set_window(self.window_size, self.max_packet_size)
+            chan._set_window(local_window_size, local_max_packet_size)
         finally:
             self.lock.release()
         self._send_user_message(m)
@@ -133,7 +165,10 @@ if paramiko and server_version_str2tuple(paramiko.__version__) >= (1, 7, 4):
     paramiko.Transport.open_channel = wba_open_channel
 else:
     print "Warning! Can't use connect with timeout in paramiko", paramiko and paramiko.__version__
-    log_warning('Cannot use connect with timeout in paramiko version %s\n' % paramiko.__version__)
+    if paramiko:
+        log_warning('Cannot use connect with timeout in paramiko version %s\n' % paramiko.__version__)
+    else:
+        log_warning('Paramiko unavailable.\n')
 
 #===============================================================================
 class ConnectionError(Exception):
@@ -310,6 +345,22 @@ class WbAdminSSH(object):
             #  raise OperationCancelledError("Cancelled login")
         log_debug2("%s: Leave\n" % self.__class__.__name__)
 
+    def _get_ssh_config_path(self):
+        paths = []
+        if platform.system().lower() == "windows":
+            paths.append("%s\ssh\config" % mforms.App.get().get_user_data_folder())
+            paths.append("%s\ssh\ssh_config" % mforms.App.get().get_user_data_folder())
+        else:
+            paths.append("~/.ssh/config")
+            paths.append("~/.ssh/ssh_config")
+            
+        for path in paths:
+            if os.path.isfile(os.path.expanduser(path)):
+                return os.path.expanduser(path)
+        else:
+            log_debug3("ssh config file not found")
+            return None
+
     def connect(self, host, port, user, pwd, usekey = 0, key = None):
         if port == None or port == 0:
             port = 22
@@ -320,22 +371,49 @@ class WbAdminSSH(object):
         if key and key.startswith("~"):
             key = os.path.expanduser(key)
 
+        config = paramiko.config.SSHConfig()
+        config_file_path = self._get_ssh_config_path()
+        if config_file_path:
+            with open(config_file_path) as f:
+                config.parse(f)
+
+        opts = config.lookup(host)
+        
+
         client = paramiko.SSHClient()
         if usekey:
-            client.load_system_host_keys()
+            ssh_known_hosts_file = None
+            if "userknownhostsfile" in opts:
+                ssh_known_hosts_file = opts["userknownhostsfile"]
+            else:
+                client.get_host_keys().clear()
+                ssh_known_hosts_file = '~/.ssh/known_hosts'
+                
+                if platform.system().lower() == "windows":
+                    ssh_known_hosts_file = '%s\ssh\known_hosts' % mforms.App.get().get_user_data_folder()
+                    
+            try:
+                client.load_host_keys(os.path.expanduser(ssh_known_hosts_file))
+            except IOError, e:
+                log_warning("IOError, probably caused by file %s not found, the message was: %s\n" % (ssh_known_hosts_file, e))
+        
+        if "stricthostkeychecking" in opts and opts["stricthostkeychecking"].lower() == "no":
+            client.set_missing_host_key_policy(WarningPolicy())
+        else:
+            client.set_missing_host_key_policy(StoreIfConfirmedPolicy())
 
         # TODO: Check if file with
         #client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
-        client.set_missing_host_key_policy(paramiko.WarningPolicy()) # This is a temp workaround, later we need to have UI with buttons accept
+        #client.set_missing_host_key_policy(paramiko.WarningPolicy()) # This is a temp workaround, later we need to have UI with buttons accept
 
            
         try:
             if 'timeout' in paramiko.SSHClient.connect.func_code.co_varnames:
-                client.connect(hostname = host, port = int(port), username = user, password = pwd, pkey = None,
-                                    key_filename = key, timeout = 10, look_for_keys=False, allow_agent=bool(usekey) )
+                client.connect(hostname = host, port = int(port), username = user, password = pwd, #pkey = None,
+                                    key_filename = key, timeout = 10, look_for_keys=bool(usekey), allow_agent=bool(usekey) )
             else:
-                client.connect(hostname = host, port = int(port), username = user, password = pwd, pkey = None,
-                                    key_filename = key, look_for_keys=False, allow_agent=bool(usekey) )
+                client.connect(hostname = host, port = int(port), username = user, password = pwd, #pkey = None,
+                                    key_filename = key, look_for_keys=bool(usekey), allow_agent=bool(usekey) )
             log_info("%s: Connected via ssh to %s\n" % (self.__class__.__name__, host) )
             if self.keepalive != 0:
                 client.get_transport().set_keepalive(self.keepalive)
@@ -353,6 +431,9 @@ class WbAdminSSH(object):
             #  raise ConnectionError("Could not establish SSH connection: %r.\nMake sure the SSH daemon is running and is accessible." % exc)
             #else:
             #  raise ConnectionError("Could not establish SSH connection: %r.\nMake sure the SSH daemon is running and is accessible." % exc)
+        except paramiko.BadHostKeyException, exc:
+            raise Exception(format_bad_host_exception(exc, '%s\ssh\known_hosts' % mforms.App.get().get_user_data_folder() if platform.system().lower() == "windows" else "~/.ssh/known_hosts file"))
+
         except paramiko.PasswordRequiredException, exc:
             if pwd is not None:
                 raise ConnectionError("Could not unlock private keys. %s" % exc)
@@ -368,11 +449,16 @@ class WbAdminSSH(object):
         except paramiko.AuthenticationException, exc:
             import traceback
             log_error("Error opening SSH connection: %s\n" % traceback.format_exc())
-            raise InvalidPasswordError("Invalid password for SSH user %s" % user)
+            if usekey:
+                raise InvalidPasswordError("Invalid ssh key %s for SSH user %s" % (key ,user))
+            else:
+                raise InvalidPasswordError("Invalid password for SSH user %s" % user)
         except paramiko.SSHException, exc:
             import traceback
             log_error("Error opening SSH connection: %s\n" % traceback.format_exc())
             raise ConnectionError("Could not establish SSH connection: %s." % exc)
+        except SSHFingerprintNewError, exc: # We need to handle this upper in the chain
+            raise exc
         except Exception, exc:
             import traceback
             log_error("Error opening SSH connection: %s\n" % traceback.format_exc())
@@ -675,7 +761,8 @@ class WbAdminSSH(object):
                 spawn_process = False
                 if 'nohup' in cmd:
                     spawn_process = True
-
+                
+                chan = None
                 chan, stdin, stdout, stderr = self._prepare_channel(transport)
                 wait_output = CmdOutput.WAIT_ALWAYS
                 if options and options.has_key(CmdOptions.CMD_WAIT_OUTPUT):
