@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -29,6 +29,7 @@ import time
 import inspect
 import random
 import string
+import functools
 
 # Declares the global variable for sudo prefix
 default_sudo_prefix = ''
@@ -87,7 +88,6 @@ def wrap_for_sudo(command, sudo_prefix, as_user = Users.ADMIN, to_spawn = False)
         if as_user != Users.ADMIN:
             sudo_user = "sudo -u %s" % as_user
             sudo_prefix = sudo_prefix.replace('sudo', sudo_user)
-
         if '/bin/sh' in sudo_prefix or '/bin/bash' in sudo_prefix:
             command = sudo_prefix + " \"" + command.replace('\\', '\\\\').replace('"', r'\"').replace('$','\\$') + "\""
         else:
@@ -97,6 +97,29 @@ def wrap_for_sudo(command, sudo_prefix, as_user = Users.ADMIN, to_spawn = False)
 
 ###
 
+# Function decorator for absolute path validation
+def useAbsPath(param):
+    def abs_path_validator(function):
+        @functools.wraps(function)
+        def decorated_function(*args, **kw):
+            path_index = None
+            try:
+                path_index = function.__code__.co_varnames.index(param)
+            except ValueError:
+                # this implies an internal coding error and will just be logged
+                # It means the programmer indicated to validate an unexisting parameter
+                log_warning ('Error on path validation for function %s, using invalid parameter "%s"\n' % (function.__name__, param))
+            
+            # Only performs the actual validation when a valid parameter is set
+            if path_index is not None:
+                path_value = args[path_index]
+                
+                if type(path_value) not in [str, unicode] or not os.path.isabs(path_value):
+                    raise ValueError('Error on path validation for function "%s", parameter "%s" must be an absolute path: %s' % (function.__name__, param, path_value))
+                
+            return function(*args, **kw)
+        return decorated_function
+    return abs_path_validator
 
 
 class SSH(WbAdminSSH):
@@ -215,19 +238,28 @@ def local_run_cmd_linux(command, as_user = Users.CURRENT, user_password=None, su
         return data
 
     # script should already have sudo
-    child = subprocess.Popen(["/bin/bash", "-c", script], bufsize=0,
+    my_env = os.environ.copy()
+    my_env['LANG'] = "C" # Force english locale 
+    child = subprocess.Popen(["/bin/sh", "-c", script], bufsize=0,
                              stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
-                             close_fds=True)
+                             close_fds=True, env = my_env)
 
     expect_sudo_failure = False
-    if as_user != Users.CURRENT:
+
+    if as_user != Users.CURRENT: 
         # If sudo is being used, we need to input the password
         data = read_nonblocking_until_nl_or(child, child.stdout, "EnterPasswordHere", timeout=output_timeout)
         if data.endswith("EnterPasswordHere"):
+            
+            if not user_password:
+                log_debug2("Password required for sudo, but user_password is empty, throwing exception.")
+                child.terminate() # sudo need pw, but pw is empty, throw exception, 
+                                  # it will be handled upper in the chain and proper pw dialog will be shown.
+                raise InvalidPasswordError("Incorrect password for sudo")
             # feed the password
             if debug_run_cmd:
                 log_debug2("local_run_cmd_linux: sending password to child...\n")
-            child.stdin.write((user_password or "")+"\n")
+            child.stdin.write((user_password or " ")+"\n"+'\x1a')
             expect_sudo_failure = True # we could get a Sorry or the password prompt again
         else:
             # If the prompt didn't come in, it could mean that password is not required
@@ -239,7 +271,7 @@ def local_run_cmd_linux(command, as_user = Users.CURRENT, user_password=None, su
             # - a certain number of lines are read from the pipe
             # - until a timeout occurs (a short one, since the banner shouldn't take very long to get printed)
             # - until the sudo terminates
-
+            log_debug2("sudo prompt available, but it's not standard. Trying to parse.")
             max_lines_to_read_until_giving_up_waiting_for_sudo_prompt = 10
             num_seconds_to_wait_for_sudo_greeting_message_until_we_assume_prompt_wont_come = 1
 
@@ -248,6 +280,11 @@ def local_run_cmd_linux(command, as_user = Users.CURRENT, user_password=None, su
             while child.poll() is None:
                 data = read_nonblocking_until_nl_or(child, child.stdout, "EnterPasswordHere", timeout=1)
                 if data.endswith("EnterPasswordHere"):
+                    if not user_password:
+                        log_debug2("Password required for sudo, but user_password is empty, throwing exception.")
+                        child.terminate() # sudo need pw, but pw is empty, throw exception, 
+                                          # it will be handled upper in the chain and proper pw dialog will be shown.
+                        raise InvalidPasswordError("Incorrect password for sudo")
                     log_info("Banner message from sudo for command %s:\n%s\n" % (script, "".join(buffered_output)))
                     buffered_output = None
                     # ok, so the stuff that came in until now is all garbage and the sudo prompt finally arrived
@@ -291,7 +328,6 @@ def local_run_cmd_linux(command, as_user = Users.CURRENT, user_password=None, su
             if output_handler and current_text:
                 output_handler(current_text)
 
-    
     # Try to read anything left, wait exit
     try:
         current_text, _ = child.communicate()
@@ -299,7 +335,6 @@ def local_run_cmd_linux(command, as_user = Users.CURRENT, user_password=None, su
             output_handler(current_text)
     except:
         pass
-    
     result = child.returncode
     if debug_run_cmd:
         log_debug2("local_run_cmd_linux: child returned %s\n" % result)
@@ -336,7 +371,7 @@ def local_run_cmd_windows(command, as_user=Users.CURRENT, user_password=None, su
 
             # Patch to ensure old OS calls get executed properly
             actual_command = command.split(' ')[0]
-            if not actual_command in ['LISTDIR','GETFILE','GET_FREE_SPACE', 'CHECK_DIR_WRITABLE', 'CHECK_PATH_EXISTS', 'CREATE_DIRECTORY', 'CREATE_DIRECTORY_RECURSIVE', 'REMOVE_DIRECTORY', 'REMOVE_DIRECTORY_RECURSIVE','DELETE_FILE', 'COPY_FILE', 'GET_FILE_OWNER', 'GETFILE_LINES', 'EXEC']:
+            if not actual_command in ['LISTDIR','GETFILE','GET_FREE_SPACE', 'CHECK_FILE_READABLE', 'CHECK_DIR_WRITABLE', 'CHECK_PATH_EXISTS', 'CREATE_DIRECTORY', 'CREATE_DIRECTORY_RECURSIVE', 'REMOVE_DIRECTORY', 'REMOVE_DIRECTORY_RECURSIVE','DELETE_FILE', 'COPY_FILE', 'GET_FILE_OWNER', 'GETFILE_LINES', 'EXEC']:
                 command = "EXEC " + command
             cmdname = helper_path
             cmdparams = '%d %s %s %s' % (listener.port, listener.handshake, listener.close_key, command)
@@ -790,6 +825,9 @@ class FileOpsNope(object):
     def _copy_file(self, source, dest, as_user = Users.CURRENT, user_password = None): # not used externally
         pass
 
+    def check_file_readable(self, path, as_user=Users.CURRENT, user_password=None):
+        return False
+
     def check_dir_writable(self, path, as_user=Users.CURRENT, user_password=None):
         return False
 
@@ -824,8 +862,9 @@ class FileOpsLinuxBase(object):
         
     # Exception Handling will vary on local and remote
     def raise_exception(self, message, custom_messages = {}):
-        pass
+        raise Exception(message)
         
+    @useAbsPath("filename")
     def file_exists(self, filename, as_user=Users.CURRENT, user_password=None):
         res = self.process_ops.exec_cmd('test -e ' + quote_path(filename),
                             as_user,
@@ -834,6 +873,7 @@ class FileOpsLinuxBase(object):
                             options={CmdOptions.CMD_WAIT_OUTPUT:CmdOutput.WAIT_NEVER})
         return res == 0
     
+    @useAbsPath("path")
     def get_available_space(self, path, as_user=Users.CURRENT, user_password=None):
         output = StringIO.StringIO()
         res = self.process_ops.exec_cmd("LC_ALL=C df -Ph %s" % quote_path(path),
@@ -850,6 +890,7 @@ class FileOpsLinuxBase(object):
         
         return available
         
+    @useAbsPath("path")
     def get_file_owner(self, path, as_user = Users.CURRENT, user_password = None):
         if self.target_os == wbaOS.linux:
           command = 'LC_ALL=C stat -c %U '
@@ -869,11 +910,16 @@ class FileOpsLinuxBase(object):
             self.raise_exception(output)
         
         return output      
-        
+    
+    @useAbsPath("path")
     def create_directory(self, path, as_user = Users.CURRENT, user_password = None, with_owner=None):
         output = StringIO.StringIO()
         if with_owner:
-            command = "/bin/mkdir %s && chown %s %s" % (quote_path(path), with_owner, quote_path(path))
+            # Chown is usually restricted to the root user
+            if as_user == Users.CURRENT:
+                raise PermissionDeniedError("Cannot set owner of directory %s" % path)        
+            else:
+                command = "/bin/mkdir %s && chown %s %s" % (quote_path(path), with_owner, quote_path(path))
         else:
             command = "/bin/mkdir %s" % (quote_path(path))
             
@@ -886,7 +932,8 @@ class FileOpsLinuxBase(object):
         if res != 0:
             output = sanitize_sudo_output(output.getvalue()).strip()
             self.raise_exception(output)      
-                
+    
+    @useAbsPath("path")
     def create_directory_recursive(self, path, as_user = Users.CURRENT, user_password = None, with_owner=None):
         head, tail = splitpath(path)
         if not tail:
@@ -900,6 +947,7 @@ class FileOpsLinuxBase(object):
 
         self.create_directory(path, as_user, user_password, with_owner)
 
+    @useAbsPath("path")
     def remove_directory(self, path, as_user = Users.CURRENT, user_password = None):
         output = StringIO.StringIO()
         res = self.process_ops.exec_cmd('/bin/rmdir ' + quote_path(path),
@@ -912,6 +960,7 @@ class FileOpsLinuxBase(object):
             output = sanitize_sudo_output(output.getvalue()).strip()
             self.raise_exception(output)
 
+    @useAbsPath("path")
     def remove_directory_recursive(self, path, as_user = Users.CURRENT, user_password = None):
         output = StringIO.StringIO()
         res = self.process_ops.exec_cmd('/bin/rm -R ' + quote_path(path),
@@ -924,6 +973,7 @@ class FileOpsLinuxBase(object):
             output = sanitize_sudo_output(output.getvalue()).strip()
             self.raise_exception(output)
 
+    @useAbsPath("path")
     def delete_file(self, path, as_user = Users.CURRENT, user_password = None):
         output = StringIO.StringIO()
         res = self.process_ops.exec_cmd("/bin/rm " + quote_path(path),
@@ -936,6 +986,7 @@ class FileOpsLinuxBase(object):
             output = sanitize_sudo_output(output.getvalue()).strip()
             self.raise_exception(output)
 
+    @useAbsPath("filename")
     def get_file_content(self, filename, as_user = Users.CURRENT, user_password = None, skip_lines=0): # may raise IOError
         command = ''
         output = StringIO.StringIO()
@@ -958,8 +1009,21 @@ class FileOpsLinuxBase(object):
 
         return output
         
+    @useAbsPath("path")
+    def _create_file(self, path, content):
+        try:
+            f = open(path, 'w')
+            f.write(content)
+            f.close()
+        except (IOError, OSError), err:
+            if err.errno == errno.EACCES:
+                raise PermissionDeniedError("Could not open file %s for writing" % path)
+            raise err
+        
+        
     def _copy_file(self, source, dest, as_user = Users.CURRENT, user_password = None):
         output = StringIO.StringIO()
+        
         res = self.process_ops.exec_cmd("LC_ALL=C /bin/cp " + quote_path(source) + " " + quote_path(dest),
                       as_user   = as_user,
                       user_password = user_password,
@@ -969,7 +1033,37 @@ class FileOpsLinuxBase(object):
         if res != 0:
             output = sanitize_sudo_output(output.getvalue()).strip()
             self.raise_exception(output)
+    
+    @useAbsPath("path")
+    def check_file_readable(self, path, as_user=Users.CURRENT, user_password=None):
+        ret_val = True
         
+        output = StringIO.StringIO()
+        
+        path = quote_path(path)
+        command = "test -e %s;_fe=$?;test -f %s;_fd=$?;test -r %s;echo $_fe$_fd$?" % (path, path, path)
+        self.process_ops.exec_cmd(command,
+                                  as_user,
+                                  user_password,
+                                  output_handler = output.write)
+
+        # The validation will depend on the output and not the returned value
+        output = sanitize_sudo_output(output.getvalue()).strip()
+      
+        log_debug2('check_file_readable :%s %s\n' % (len(output), output))
+        
+        if len(output) == 3:
+            if output[0] == '1':
+                raise OSError(errno.ENOENT, 'The path "%s" does not exist' % path)
+            elif output[1] == '1':
+                raise OSError(errno.ENOTDIR, 'The path "%s" is not a regular file' % path)
+            elif output[2] == '1':
+                ret_val = False
+        else:
+          raise Exception('Unable to verify file is readable : %s' % output)
+
+        return ret_val    
+    @useAbsPath("path")
     def check_dir_writable(self, path, as_user=Users.CURRENT, user_password=None):
         ret_val = True
         
@@ -998,14 +1092,15 @@ class FileOpsLinuxBase(object):
           raise Exception('Unable to verify directory is writable : %s' % output)
 
         return ret_val        
-        
+    
+    @useAbsPath("path")
     def listdir(self, path, as_user = Users.CURRENT, user_password = None, include_size=False): 
         file_list = []
         if include_size:
             # for ls -l, the output format changes depending on stdout being a terminal or not
             # since both cases are possible, we need to handle both at the same time (1st line being total <nnnn> or not)
             # the good news is that if the line is there, then it will always start with total, regardless of the locale
-            command = 'LC_ALL=C /bin/ls -l -p %s | awk \'!/^total/ { print $5,$NF }\' ; exit ${PIPESTATUS[0]}' % quote_path(path)
+            command = 'LC_ALL=C /bin/ls -l -p %s' % quote_path(path)
         else:
             command = 'LC_ALL=C /bin/ls -1 -p %s' % quote_path(path)
             
@@ -1015,6 +1110,7 @@ class FileOpsLinuxBase(object):
                                         user_password,
                                         output_handler = output.write)
         output = sanitize_sudo_output(output.getvalue().strip())
+        
         if res != 0:
             custom_messages = {
                                 errno.ENOENT:"The path \"%s\" does not exist" % path,
@@ -1026,7 +1122,7 @@ class FileOpsLinuxBase(object):
         else:
             try:
                 if include_size:
-                    file_list = [(f, int(s)) for s, f in [s.strip().split(" ", 1) for s in output.split("\n")]]
+                    file_list = [(f, int(s)) for _, _, _, _, s, _, _, _, f in [s.strip().split(None, 8) for s in output.split("\n") if not s.startswith('total')]]
                 else:
                     file_list = [s.strip() for s in output.split("\n")]
                     
@@ -1042,8 +1138,23 @@ class FileOpsLinuxBase(object):
         pass
         
     def _create_temp_file(self, content):
-        pass
+        tmp = tempfile.NamedTemporaryFile(delete = False)
+        tmp_name = tmp.name
 
+        try:
+            log_debug('%s: Writing file contents to tmp file "%s"\n' %  (self.__class__.__name__, tmp_name) )
+            tmp.write(content)
+            tmp.flush()
+        except Exception, exc:
+            log_error('%s: Exception caught: %s\n' % (self.__class__.__name__, str(exc)) )
+            if tmp:
+                tmp.close()
+            raise
+            
+        return tmp_name
+        
+
+    @useAbsPath("filename")
     def save_file_content_and_backup(self, filename, content, backup_extension, as_user = Users.CURRENT, user_password = None, mode = None):
         log_debug('%s: Saving file "%s" with backup (sudo="%s")\n' % (self.__class__.__name__, filename, str(as_user)) )
         # Checks if the target folder is writable
@@ -1053,6 +1164,7 @@ class FileOpsLinuxBase(object):
             # Creates a backup of the existing file... if any
             if backup_extension and self.file_exists(filename, as_user, user_password):
                 log_debug('%s: Creating backup of "%s" to "%s"\n' %  (self.__class__.__name__, filename, filename+backup_extension))
+                
                 self._copy_file(source = filename, dest = filename + backup_extension,
                                 as_user = as_user, user_password = user_password)
                                     
@@ -1062,6 +1174,7 @@ class FileOpsLinuxBase(object):
                 
                          
                 log_debug('%s: Copying over tmp file to final filename using sudo: %s -> %s\n' % (self.__class__.__name__, temp_file, filename) )
+                
                 self._copy_file(source = temp_file, dest = filename, 
                                 as_user = Users.ADMIN, user_password = user_password)
                     
@@ -1130,37 +1243,12 @@ class FileOpsLocalUnix(FileOpsLinuxBase):
         raise Exception(custom_messages.get(None, message))
         
     # content must be a string
+    @useAbsPath("filename")
     def save_file_content(self, filename, content, as_user = Users.CURRENT, user_password = None, mode = None):
         self.save_file_content_and_backup(filename, content, None, as_user, user_password, mode)
 
-    def _create_file(self, path, content):
-        try:
-            f = open(path, 'w')
-            f.write(content)
-            f.close()
-        except (IOError, OSError), err:
-            if err.errno == errno.EACCES:
-                raise PermissionDeniedError("Could not open file %s for writing" % path)
-            raise err
-
-    def _create_temp_file(self, content):
-        tmp = tempfile.NamedTemporaryFile(delete = False)
-        tmp_name = tmp.name
-
-        try:
-            log_debug('%s: Writing file contents to tmp file "%s"\n' %  (self.__class__.__name__, tmp_name) )
-            tmp.write(content)
-            tmp.flush()
-        except Exception, exc:
-            log_error('%s: Exception caught: %s\n' % (self.__class__.__name__, str(exc)) )
-            if tmp:
-                tmp.close()
-            raise
-            
-        return tmp_name
-
-
     # UseCase: If get_file_content fails with exception of access, try sudo
+    @useAbsPath("filename")
     def get_file_content(self, filename, as_user = Users.CURRENT, user_password = None, skip_lines=0):
         if as_user == Users.CURRENT:
             try:
@@ -1193,25 +1281,8 @@ class FileOpsLocalUnix(FileOpsLinuxBase):
 
         return cont
 
-
-    def create_directory(self, path, as_user = Users.CURRENT, user_password = None, with_owner=None):
-        if as_user == Users.CURRENT:
-            if with_owner is not None:
-                raise PermissionDeniedError("Cannot set owner of directory %s" % path)
-                
-            FileUtils.create_directory(path)
-        else:
-            FileOpsLinuxBase.create_directory(self, path, as_user, user_password, with_owner)
-
-    def create_directory_recursive(self, path, as_user = Users.CURRENT, user_password = None, with_owner=None):
-        if as_user == Users.CURRENT:
-            if with_owner is not None:
-                raise PermissionDeniedError("Cannot set owner of directory %s" % path)
-                
-            FileUtils.create_directory_recursive(path)
-        else:
-            FileOpsLinuxBase.create_directory_recursive(self, path, as_user, user_password, with_owner)
-
+  
+    @useAbsPath("path")
     def delete_file(self, path, as_user = Users.CURRENT, user_password = None):
         if as_user == Users.CURRENT:
             FileUtils.delete_file(path)                   
@@ -1226,6 +1297,7 @@ class FileOpsLocalUnix(FileOpsLinuxBase):
             FileOpsLinuxBase._copy_file(self, source, dest, as_user, user_password)
 
     # Return format is list of entries in dir (directories go first, each dir name is followed by /)
+    @useAbsPath("path")
     def listdir(self, path, as_user = Users.CURRENT, user_password = None, include_size=False): # base operation to build file_exists and remote file selector
         file_list = []
         if as_user == Users.CURRENT: 
@@ -1309,6 +1381,14 @@ class FileOpsLocalWindows(object): # Used for remote as well, if not using sftp
 
         return ret_val
 
+    def check_file_readable(self, path, as_user=Users.CURRENT, user_password=None):
+        if as_user == Users.CURRENT:
+            ret_val = FileUtils.check_file_readable(path)
+        else:
+            ret_val = self.exec_helper_command('CHECK_FILE_READABLE %s' % path, FunctionType.Boolean, as_user, user_password)
+          
+        return ret_val
+
 
     def check_dir_writable(self, path, as_user=Users.CURRENT, user_password=None):
         if as_user == Users.CURRENT:
@@ -1755,6 +1835,12 @@ class FileOpsRemoteWindows(object):
             print "Attempt to read remote file with no ssh session"
             raise Exception("Cannot read remote file without an SSH session")
 
+    #TODO fix for windows
+    def check_file_readable(self, path, as_user=Users.CURRENT, user_password=None):
+        msg, code = self.process_ops.get_cmd_output('if exist ' + quote_path(path) + ' ( type '+ quote_path(path) +'>NUL && echo 0 ) else ( echo 1 )')
+        ret = (code == 0)
+        return ret
+
     def check_dir_writable(self, path, as_user=Users.CURRENT, user_password=None):
         msg, code = self.process_ops.get_cmd_output('echo 1 > ' + quote_path(path + "/wba_tmp_file.bak"))
         ret = (code == 0)
@@ -1799,6 +1885,7 @@ class ServerManagementHelper(object):
         self.profile = profile
 
         klass = None
+        
         match_tuple = (profile.host_os, profile.target_os, profile.connect_method)
         for k in _process_ops_classes:
             if k.match(match_tuple):
@@ -1833,6 +1920,10 @@ class ServerManagementHelper(object):
             return self.shell.cmd_output_encoding
         return ""
 
+    #-----------------------------------------------------------------------------
+    def check_file_readable(self, path, as_user=Users.CURRENT, user_password=None):
+        return self.file.check_file_readable(path, as_user, user_password)
+
     #-----------------------------------------------------------------------------
     def check_dir_writable(self, path, as_user=Users.CURRENT, user_password=None):
         return self.file.check_dir_writable(path, as_user, user_password)
@@ -1982,41 +2073,71 @@ class SFTPInputFile(object):
 
 import multiprocessing
 class SudoTailInputFile(object):
-    def __init__(self, server_helper, path, password):
+    def __init__(self, server_helper, path, password = None, password_cb = None):
         self.path = path
         self.server_helper = server_helper # ServerManagementHelper
         self.data = None
         self._password = password
+        self._password_cb = password_cb
         self.skip_first_newline = True
         self._pos = 0
         self._proc = None
         self._queue = None
         # multiprocessing + paramiko doesn't work, so fallback to threading for remote access
         self._is_local = self.server_helper.profile.is_local
-
+        # check if we can read the file as normall user, if not we need to gather password if callback is provided
+        self._need_sudo = False
+        while True:
+            if not self._need_sudo:
+                try:
+                    if not self.server_helper.check_file_readable(self.path):
+                        self._need_sudo = True 
+                    break
+                except OSError, e:
+                    log_debug3("check_file_readable returned OSError, we will try with sudo then")
+                    self._need_sudo = True
+            else:
+                self._password = self.get_password
+                self.server_helper.check_file_readable(self.path, as_user = Users.ADMIN, user_password=self._password)
+                break
+            
     def __del__(self):
         if self._proc:
             self._proc.join()
 
     def tell(self):
         return self._pos
+    
+    @property
+    def get_password(self):
+        return self._password if not self._password_cb else self._password_cb()
 
     @property
     def size(self):
-        files = self.server_helper.listdir(self.path, as_user = Users.ADMIN, user_password=self._password, include_size=True)
+        if not self._need_sudo:
+            files = self.server_helper.listdir(self.path, as_user = Users.CURRENT, user_password=None, include_size=True)
+        else:
+            files = self.server_helper.listdir(self.path, as_user = Users.ADMIN, user_password=self._password, include_size=True)
         if not files:
             raise RuntimeError("Could not get size of file %s" % self.path)
         return files[0][1]
 
     def get_range(self, start, end):
         f = StringIO.StringIO()
-        ret = self.server_helper.execute_command("/bin/dd if=%s ibs=1 skip=%i count=%i 2> /dev/null" % (quote_path(self.path), start, end-start), as_user = Users.ADMIN, user_password=self._password, output_handler=f.write)
+        if not self._need_sudo:
+            ret = self.server_helper.execute_command("/bin/dd if=%s ibs=1 skip=%i count=%i 2> /dev/null" % (quote_path(self.path), start, end-start), as_user = Users.CURRENT, user_password=None, output_handler=f.write)
+        else:
+            ret = self.server_helper.execute_command("/bin/dd if=%s ibs=1 skip=%i count=%i 2> /dev/null" % (quote_path(self.path), start, end-start), as_user = Users.ADMIN, user_password=self.get_password, output_handler=f.write)
+
         if ret != 0:
             raise RuntimeError("Could not get data from file %s" % self.path)
         return f.getvalue()
 
     def read_task(self, offset, file):
-        self.server_helper.execute_command("/bin/dd if=%s ibs=1 skip=%i 2> /dev/null" % (quote_path(self.path), offset), as_user = Users.ADMIN, user_password=self._password, output_handler=file.write)
+        if not self._need_sudo:
+            self.server_helper.execute_command("/bin/dd if=%s ibs=1 skip=%i 2> /dev/null" % (quote_path(self.path), offset), as_user = Users.CURRENT, user_password=None, output_handler=file.write)
+        else:
+            self.server_helper.execute_command("/bin/dd if=%s ibs=1 skip=%i 2> /dev/null" % (quote_path(self.path), offset), as_user = Users.ADMIN, user_password=self.get_password, output_handler=file.write)
         # this will signal the reader end that there's no more data
         file.close()
 
@@ -2041,7 +2162,10 @@ class SudoTailInputFile(object):
             return self.start_read_task_from(offset)
         self._pos = offset
         f = StringIO.StringIO()
-        self.server_helper.execute_command("/bin/dd if=%s ibs=1 skip=%i 2> /dev/null" % (quote_path(self.path), offset), as_user = Users.ADMIN, user_password=self._password, output_handler=f.write)
+        if not self._need_sudo:
+            self.server_helper.execute_command("/bin/dd if=%s ibs=1 skip=%i 2> /dev/null" % (quote_path(self.path), offset), as_user = Users.CURRENT, user_password=None, output_handler=f.write)
+        else:
+            self.server_helper.execute_command("/bin/dd if=%s ibs=1 skip=%i 2> /dev/null" % (quote_path(self.path), offset), as_user = Users.ADMIN, user_password=self._password, output_handler=f.write)
         self.data = f
         self.data.seek(0)
         if self.skip_first_newline:
