@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -15,15 +15,16 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 # 02110-1301  USA
 
-from mforms import newButton, newPanel, newLabel, newBox, newCheckBox, newTable, newTextBox, newImageBox, Utilities
+from mforms import newButton, newLabel, newBox, newCheckBox, newTextBox, Utilities
 import mforms
 
-from wb_common import dprint_ex
+from wb_common import dprint_ex, OperationCancelledError
 import datetime
 from wb_admin_utils import no_remote_admin_warning_label, make_panel_header
 
 from wb_log_reader import ErrorLogFileReader
 
+
 class WbAdminConfigurationStartup(mforms.Box):
     long_status_msg = None
     short_status_msg = None
@@ -43,19 +44,24 @@ class WbAdminConfigurationStartup(mforms.Box):
     def identifier(cls):
         return "admin_start_stop"
 
+    #---------------------------------------------------------------------------
+    def print_output_cb(self, text):
+        self.ctrl_be.uitask(self.print_output, text)
+
     #---------------------------------------------------------------------------
     def print_output(self, text):
         ts = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S - ")
-        self.startup_msgs_log.append_text_with_encoding(ts + text + "\n", self.ctrl_be.server_helper.cmd_output_encoding)
+        if self.startup_msgs_log:
+            self.startup_msgs_log.append_text_with_encoding(ts + text + "\n", self.ctrl_be.server_helper.cmd_output_encoding)
 
     #---------------------------------------------------------------------------
     def __init__(self, ctrl_be, server_profile, main_view):
         mforms.Box.__init__(self, False)
         self.set_managed()
-        self.set_release_on_add()                               
+        self.set_release_on_add()
         self.main_view = main_view
         self.ctrl_be = ctrl_be
-        self.server_profile = server_profile
+        self.ctrl_be.server_profile = server_profile
 
     #---------------------------------------------------------------------------
     @property
@@ -123,13 +129,14 @@ class WbAdminConfigurationStartup(mforms.Box):
         self.add(heading, False, True)
 
         self.startup_msgs_log = newTextBox(mforms.BothScrollBars)
+        self.startup_msgs_log.set_name('StartupMessagesLog')
         self.startup_msgs_log.set_read_only(True)
         self.add(self.startup_msgs_log, True, True)
 
         button_box = newBox(True)
         self.refresh_button = newButton()
         self.refresh_button.set_text("Refresh Status")
-        self.refresh_button.add_clicked_callback(lambda:self.refresh(2))
+        self.refresh_button.add_clicked_callback(lambda:self.refresh(True))
         button_box.add(self.refresh_button, False, False)
 
         self.copy_to_clipboard_button = newButton()
@@ -152,22 +159,32 @@ class WbAdminConfigurationStartup(mforms.Box):
 
     #---------------------------------------------------------------------------
     def page_activated(self):
+        if not self.server_profile.admin_enabled:
+            self.add(no_remote_admin_warning_label(self.server_profile), True, True)
+            return
+        
         if not self.ui_created:
             self.create_ui()
             self.ui_created = True
+
         if self.server_control:
-            self.server_control.set_output_handler(self.print_output)
-        self.refresh()
+            self.server_control.set_output_handler(self.print_output_cb)
+
+        if self.is_server_running_prev_check is None:
+            self.is_server_running_prev_check = self.ctrl_be.is_server_running()
+            self.update_ui(self.is_server_running_prev_check)
+            self.print_new_error_log_entries()
+
 
     #---------------------------------------------------------------------------
     def server_started_event(self):
         dprint_ex(2, "Handling server start event in start/stop page")
-        self.ctrl_be.uitask(self.update_ui, "running")
+        self.update_ui("running")
 
     #---------------------------------------------------------------------------
     def server_stopped_event(self):
         dprint_ex(2, "Handling server stop event in start/stop page")
-        self.ctrl_be.uitask(self.update_ui, "stopped")
+        self.update_ui("stopped")
 
     #---------------------------------------------------------------------------
     def update_ui(self, server_status):
@@ -215,6 +232,10 @@ class WbAdminConfigurationStartup(mforms.Box):
             try:
                 self.error_log_reader = ErrorLogFileReader(self.ctrl_be, self.server_profile.error_log_file_path)
                 self.error_log_position = self.error_log_reader.file_size
+            except OperationCancelledError, e:
+                self.startup_msgs_log.append_text_with_encoding("Cancelled password input to open error log file: %s\n" % e,
+                                                              self.ctrl_be.server_helper.cmd_output_encoding)
+                raise
             except Exception, e:
                 self.startup_msgs_log.append_text_with_encoding("Could not open error log file: %s\n" % e,
                                                                 self.ctrl_be.server_helper.cmd_output_encoding)
@@ -237,7 +258,13 @@ class WbAdminConfigurationStartup(mforms.Box):
 
     #---------------------------------------------------------------------------
     def start_stop_clicked(self):
-        self.start_error_log_tracking()
+        try:
+            self.start_error_log_tracking()
+        except OperationCancelledError:
+            # we could abort everything if we knew that start/stop server also needs sudo password
+            # to avoid user having to cancel that twice, but since we're not sure if the start/stop will
+            # indeed require the sudo password, we can't give up yet
+            pass
         status = self.ctrl_be.is_server_running(verbose=1)
         # Check if server was started/stoped from outside
         if self.is_server_running_prev_check == status:
@@ -246,7 +273,7 @@ class WbAdminConfigurationStartup(mforms.Box):
                 self.refresh_button.set_enabled(False)
 
                 try:
-                    if self.server_control and not self.server_control.stop_async(self.async_stop_callback):
+                    if self.server_control and not self.server_control.stop_async(self.async_stop_callback, True):
                         self.start_stop_btn.set_enabled(True)
                         self.refresh_button.set_enabled(True)
                         return
@@ -262,7 +289,7 @@ class WbAdminConfigurationStartup(mforms.Box):
                 self.refresh_button.set_enabled(False)
 
                 try:
-                    if self.server_control and not self.server_control.start_async(self.async_start_callback):
+                    if self.server_control and not self.server_control.start_async(self.async_start_callback, True):
                         self.start_stop_btn.set_enabled(True)
                         self.refresh_button.set_enabled(True)
                         return
@@ -296,10 +323,13 @@ class WbAdminConfigurationStartup(mforms.Box):
                               "Administrator password was possibly wrong.",
                               "Retry", "Cancel", "")
             if r == mforms.ResultOk:
-                self.server_control.stop_async(self.async_stop_callback)
-                return
+                if self.server_control.stop_async(self.async_stop_callback):
+                    return
             else:
                 self.print_output("Could not stop server. Permission denied")
+        elif status == "need_password":
+            if self.server_control.stop_async(self.async_stop_callback, False):
+                return
         else:
             self.print_output("Could not stop server: %s" % (status or "unknown error"))
             Utilities.show_error("Could not stop server", str(status), "OK", "", "")
@@ -328,6 +358,9 @@ class WbAdminConfigurationStartup(mforms.Box):
                 return
             else:
                 self.print_output("Could not stop server. Permission denied")
+        elif status == "need_password":
+            self.server_control.start_async(self.async_start_callback, False)
+            return
         else:
             self.print_output("Could not start server: %s" % (status or "unknown error"))
             Utilities.show_error("Could not start server", str(status), "OK", "", "")
@@ -338,10 +371,12 @@ class WbAdminConfigurationStartup(mforms.Box):
         #self.error_log_reader = None
 
     #---------------------------------------------------------------------------
-    def refresh(self, verbose=1):
-        self.is_server_running_prev_check = self.ctrl_be.is_server_running(verbose=verbose, force_process_check=True)
-        self.update_ui(self.is_server_running_prev_check)
-        self.print_new_error_log_entries()
+    def refresh(self, verbose=False):
+        new_state = self.ctrl_be.force_check_server_state(verbose=verbose)
+        if new_state:
+            self.is_server_running_prev_check = new_state
+            self.update_ui(self.is_server_running_prev_check)
+            self.print_new_error_log_entries()
 
     #---------------------------------------------------------------------------
     def copy_to_clipboard(self):
