@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -17,13 +17,11 @@
 
 from __future__ import with_statement
 
-import threading
-import time
 import os
 import sys
+import grt
 
-from grt import log_info, log_error
-_this_file = os.path.basename(__file__)
+from workbench.log import log_info, log_error, log_debug
 
 from mforms import App, Utilities, newTabView
 import mforms
@@ -40,6 +38,9 @@ from wb_admin_variables import WbAdminVariables
 from wb_admin_security import WbAdminSecurity
 from wb_admin_logs import WbAdminLogs
 from wb_admin_export import WbAdminExport, WbAdminImport
+from wb_admin_performance_dashboard import WbAdminDashboard
+from wb_admin_perfschema_reports import WbAdminPerformanceSchema
+from wb_admin_perfschema_instrumentation import WbAdminPerformanceSchemaInstrumentation
 from wb_admin_utils import weakcb
 from wb_server_management import wbaOS
 
@@ -55,26 +56,30 @@ def scan_admin_modules():
                WbAdminSecurity,
                WbAdminVariables,
                WbAdminExport,
-               WbAdminImport]
+               WbAdminImport,
+               
+               WbAdminDashboard,
+               WbAdminPerformanceSchema,
+               WbAdminPerformanceSchemaInstrumentation]
 
     # look for extension modules
     #---------------------------------------------------------------------------
-    log_info("WBA", "Looking for extension modules for WBA...\n")
+    grt.log_info("WBA", "Looking for extension modules for WBA...\n")
     init_count = 0
     # search in the same dir where the WBA code itself is located
+    extra_mods = {}
     for location in [os.path.dirname(__file__)]:
         try:
             folders = [f for f in os.listdir(location) if f.startswith("wba_") and os.path.isdir(os.path.join(location, f))]
         except:
             continue
-
         sys.path.append(location)
 
         for candidate in folders:
             if os.path.exists(os.path.join(location, candidate, "__init__.py")):
                 mod = __import__(candidate)
                 if hasattr(mod, "wba_register"):
-                    modules.append(mod)
+                    extra_mods[candidate] = mod
                     init_count+= 1
                 else:
                     # unload the module
@@ -83,7 +88,13 @@ def scan_admin_modules():
 
         sys.path.pop()
 
-    log_info("WBA", "%i extension modules found\n" % init_count)
+    if len(extra_mods) != 0:
+        import collections
+        od = collections.OrderedDict(sorted(extra_mods.items()))
+        for mod in od.values():
+            modules.append(mod)
+        
+    grt.log_info("WBA", "%i extension modules found\n" % init_count)
 
     return modules
 
@@ -91,9 +102,6 @@ def scan_admin_modules():
 #
 #===============================================================================
 class AdministratorTab(mforms.AppView):
-    last_server_status = None
-    last_notified_server_status = None
-
     def __init__(self, ctrl_be, server_profile, main_view, editor):
         mforms.AppView.__init__(self, False, "administrator", False)
         self.editor                     = editor
@@ -107,40 +115,48 @@ class AdministratorTab(mforms.AppView):
         self.old_active_tab             = None
         self.server_profile             = server_profile
 
-        self.refresh_tasks_sleep_time   = 2
-
         # if we're in the Mac, we need to set the background color of the main view of the tab to white,
         # so that MTabSwitcher will take the cue and set the tab color to white too
         if self.server_profile.host_os == wbaOS.darwin:
             self.set_back_color("#ffffff")
-      
+
         # Setup self
         self.set_managed()
         self.set_release_on_add()
 
         self.on_close(wb_admin_utils.weakcb(self, "handle_on_close"))
 
+        nc.add_observer(self.handle_server_state_changed, "GRNServerStateChanged", editor)
+
         self.ctrl_be.add_me_for_event("server_started", self)
         self.ctrl_be.add_me_for_event("server_stopped", self)
 
         self.add(self.tabview, True, True)
 
-        Utilities.add_timeout(0.5, weakcb(self, "timeout"))
-        self.timeout_thread = threading.Thread(target = self.refresh_tasks_thread)
-        self.timeout_thread.setDaemon(True)
-        self.timeout_thread.start()
+        self._timeout_tm = Utilities.add_timeout(0.5, weakcb(self, "timeout"))
         self.tabview.add_tab_changed_callback(self.tab_changed)
 
-        self.timeout() # will call self.connect_mysql() and check if mysql is running
+        self.timeout()
+
+        # check initial state
+        if editor.isConnected:
+            self.ctrl_be.event_from_main("server_started")
 
         self.ctrl_be.continue_events() # Process events which are queue during init
-        dprint_ex(1, "WBA init complete")
+        log_debug("WBA init complete\n")
 
     #---------------------------------------------------------------------------
     def handle_on_close(self):
+        log_debug("Closing admin\n")
+        if self._timeout_tm:
+            Utilities.cancel_timeout(self._timeout_tm)
+            self._timeout_tm = None
+        nc.remove_observer(self.handle_server_state_changed)
+
         App.get().set_status_text("Closing Administator.")
         self.shutdown()
         if not self.closing:
+            log_debug("Admin close cancelled\n")
             return False
         self.ctrl_be.shutdown()
         self.release()
@@ -191,12 +207,13 @@ class AdministratorTab(mforms.AppView):
                 panel.page_activated()
             except Exception, e:
                 import traceback
-                log_error(_this_file, "Unhandled exception in Admin for %s: %s\n" % (panel, traceback.format_exc()))
+                log_error("Unhandled exception in Admin for %s: %s\n" % (panel, traceback.format_exc()))
                 mforms.Utilities.show_error("Error", "An unhandled exception occurred (%s). Please refer to the log files for details." % e, "OK", "", "")
         self.old_active_tab = panel
 
     #---------------------------------------------------------------------------
     def shutdown(self):
+        log_error("shutting down admn\n")
         dprint_ex(2, " closing")
         self.closing = True
         for tab in self.tabs:
@@ -210,21 +227,19 @@ class AdministratorTab(mforms.AppView):
         self.shutdown()
 
     #---------------------------------------------------------------------------
-    def notify_server_status_change(self, state):
-        if self.last_notified_server_status and self.last_notified_server_status != state:
-            info = { "state" : state, "connection" : self.editor.connection }
-            # this will notify the rest of the App that the server state has changed, giving them a chance
-            # to reconnect or formally disconnect
-            nc.send("GRNServerStateChanged", self.editor, info)
-        self.last_notified_server_status = state
+    def handle_server_state_changed(self, name, sender, info):
+        # called from outside when running state of the server changes
+        log_info("received notification that server state changed to %s\n" % info["state"])
+        if info["state"]:
+            self.ctrl_be.event_from_main("server_started")
+        else:
+            self.ctrl_be.event_from_main("server_stopped")
 
     #---------------------------------------------------------------------------
     def server_started_event(self):
         dprint_ex(1, "Handling start event")
         if len(self.tabs) > 0 and hasattr(self.tabs[0], 'print_output'):
             self.ctrl_be.uitask(self.tabs[0].print_output, "Server is running")
-        self.notify_server_status_change("running")
-        self.refresh_tasks_sleep_time = 2
         dprint_ex(1, "Done handling start event")
 
     #---------------------------------------------------------------------------
@@ -232,42 +247,13 @@ class AdministratorTab(mforms.AppView):
         dprint_ex(1, "Handling stop event")
         if len(self.tabs) > 0 and hasattr(self.tabs[0], "print_output"):
             self.ctrl_be.uitask(self.tabs[0].print_output, "Server is stopped")
-        self.notify_server_status_change("stopped")
-        self.refresh_tasks_sleep_time = 3
         dprint_ex(1, "Done handling stop event")
 
-    #---------------------------------------------------------------------------
-    def refresh_tasks_thread(self):
-        dprint_ex(2, "Entering refresh tasks thread")
-
-        while not self.closing:
-            status = "unknown"
-            try:
-                status = self.ctrl_be.is_server_running(verbose=0)
-            except Exception, exc:
-                import traceback
-                traceback.print_exc()
-                print "exception getting server status: %s" % exc
-
-            control_event = None
-            if self.last_server_status != status:
-                if status == "running":
-                    control_event = "server_started"
-                elif status == "stopped":
-                    control_event = "server_stopped"
-
-            if control_event:
-                self.ctrl_be.event_from_main(control_event)
-
-            dprint_ex(3, "server running", status, ", self.closing =", self.closing)
-            self.last_server_status = status
-
-            time.sleep(self.refresh_tasks_sleep_time)
-
-        dprint_ex(2, "Leaving refresh tasks thread")
-
     #---------------------------------------------------------------------------
     def timeout(self):
         if not self.closing:
             self.ctrl_be.process_ui_task_queue()
+        if self.closing:
+            self._timeout_tm = None
         return not self.closing
+
