@@ -36,18 +36,22 @@ def call_system(command, spawn, output_handler = None):
     logging.info('Executing command: %s' % command)
 
     if spawn or output_handler is None:
-        if os.fork() != 0:
-            return
-        
-        os.setpgrp()
-        
-        for i in range(0,100):
-            try:
-                os.close(i)
-            except:
-                pass
-                  
-        os.execvp("/bin/bash", ["/bin/bash", "-c", command])
+        try:
+            if os.fork() != 0:
+                return result
+            
+            os.setpgrp()
+            
+            for i in range(0,100):
+                try:
+                    os.close(i)
+                except:
+                    pass
+                      
+            os.execvp("/bin/sh", ["/bin/sh", "-c", command])
+        except OSError, e:
+            logging.error('Error on command execution: %s' % str(e))
+            result = 1
 
     else:
         child = subprocess.Popen(command, bufsize=0, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True, preexec_fn=os.setpgrp)
@@ -198,7 +202,7 @@ class MEBCommandProcessor(MEBCommand):
 
         self._commands = {}
         self._commands['BACKUP'] = MEBBackup
-        self._commands['VERSION'] = MEBVersion
+        self._commands['VERSION'] = MEBHelperVersion
         self._commands['GET_PROFILES'] = MEBGetProfiles
         self._commands['UPDATE_SCHEDULING'] = MEBUpdateScheduling
         self._commands['PROPAGATE_DATA'] = MEBPropagateSettings
@@ -610,35 +614,46 @@ class MEBUpdateScheduling(MEBCommand):
         return schedule_command
 
     def execute(self):
+        output = StringIO.StringIO()
         ret_val = 0
         if self.read_params():
         
-            # Profile data would NOT be read on DELETE actions.
             if self.change != "DELETE":
                 self.init_profile()
             
             # Unscheduling would NOT occur on NEW profiles
             command = ""
             if self.change != "NEW":
-                if self.old_fb_schedule == "1" or self.old_label != self.new_label:
+                label = self.new_label if hasattr(self, 'new_label') else self.old_label
+                message = ", renamed from %s" % self.old_label if self.old_label != label else "."
+                
+                if self.old_fb_schedule == "1" or self.old_label != label:
                     command = self.get_unschedule_command("full")
-                    ret_val = ret_val + call_system(command, False)
+                    logging.debug("Unscheduling full backup for profile: %s%s" % (label, message))
+                    ret_val = ret_val + call_system(command, False, output.write)
 
-                if self.old_ib_schedule == "1" or self.old_label != self.new_label:
+                if self.old_ib_schedule == "1" or self.old_label != label:
                     command = self.get_unschedule_command("inc")
-                    ret_val = ret_val + call_system(command, False)
+                    logging.debug("Unscheduling incremental backup for profile: %s%s" % (label, message))
+                    ret_val = ret_val + call_system(command, False, output.write)
             
             if self.new_fb_schedule == "True":
                 self.read_profile_data("full")
                 command = self.get_schedule_command("full")
-                ret_val = ret_val + call_system(command, False)
+                logging.debug("Scheduling full backup for profile: %s." % self.new_label)
+                ret_val = ret_val + call_system(command, False, output.write)
             
             if self.new_ib_schedule == "True":
                 self.read_profile_data("inc")
                 command = self.get_schedule_command("inc")
-                ret_val = ret_val + call_system(command, False)
+                logging.debug("Scheduling incremental backup for profile: %s." % self.new_label)
+                ret_val = ret_val + call_system(command, False, output.write)
         else:
             self.print_usage()
+
+        # In case of error prints the output
+        if ret_val:
+            self.write_output(output.getvalue().strip())
         
         return ret_val
 
@@ -711,15 +726,15 @@ class MEBGetProfiles(MEBCommand):
         ret_val = False
 
         if self.param_count() == 2:
-            self.meb_version = int(self.get_param(0))
+            self.meb_command = self.get_param(0)
             self.datadir = self.get_param(1)
             ret_val = True
 
         return ret_val
 
     def print_usage(self):
-        self.write_output("GET_PROFILES <meb_version> <datadir>")
-        self.write_output("WHERE : <meb_version> : is the profile version required by the meb being used at the server for backups")
+        self.write_output("GET_PROFILES <meb_command> <datadir>")
+        self.write_output("WHERE : <meb_command> : is the path to a valid MEB executable.")
         self.write_output("WHERE :               : is the path to the datadir of the server instance for which the profiles are")
         self.write_output("                        being loaded. (There could be more than one instance on the same box).")
 
@@ -747,6 +762,11 @@ class MEBGetProfiles(MEBCommand):
                 
                 # Creates a config reader for each profile
                 profile = ConfigReader(filename)
+                
+                # Verifies the profile configured command is the same as the received as parameter
+                profile_command = profile.read_value('meb_manager', 'command', False, "")
+                if profile_command != self.meb_command:
+                    profile_issues |= 16
 
                 # Verifies the datadir to ensure it belongs to the requested instance
                 profile_datadir = profile.read_value('mysqlbackup', 'datadir', False, "")
@@ -787,9 +807,13 @@ class MEBGetProfiles(MEBCommand):
                     m = profile.read_value('meb_manager', 'inc_backups_minute', False, "")
                     data['ISCHEDULE'] = '-'.join([e, f, md, wd, h, m])
                     
-                    # Gets the profile version
-                    p_version = int(profile.read_value('meb_manager', 'version', False, "0"))
-                    if p_version == 0 and self.meb_version > 0:
+                    # Profiles with version 0 could have a UUID as the value for the "includes" field
+                    # It was used to make all of the InnoDB tables NOT matching the selection criteria on initial
+                    # versions of MEB.
+                    # On MEB 3.9.0 this became invalid as an error would be generated so
+                    # If found in a profile, we need to report the invalid configuration on the UI.
+                    profile_version = int(profile.read_value('meb_manager', 'version', False, "0"))
+                    if profile_version == '0' and check_version_at_least(self.meb_command, 3, 9, 0):
                         include = profile.read_value('mysqlbackup', 'include', False, "")
                         if include:
                             expression='^[\dA-Fa-f]{8}-([\dA-Fa-f]{4}-){3}[\dA-Fa-f]{12}$'
@@ -845,14 +869,23 @@ class MEBGetProfiles(MEBCommand):
             
         return 0
 
-class MEBVersion(MEBCommand):
-    current = "3"
 
+class MEBHelperVersion(MEBCommand):
     def __init__(self, params = None, output_handler = None):
-        super(MEBVersion, self).__init__(params, output_handler)
+        super(MEBHelperVersion, self).__init__(params, output_handler)
         
     def execute(self):
-        self.write_output(self.current)
+        try:
+          import hashlib
+          
+          file = open (__file__, 'r')
+          data = file.read()
+          md5 = hashlib.md5(data)
+          
+          self.write_output(md5.digest())
+        except Exception, e:
+            logging.error('MEBHelperVersion error ' % str(e))
+            return 1
         return 0
     
 if __name__ == "__main__":
